/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/** La classe MathGraphDataInputStream ve servir  charger les flus de donnes en C++
// Elle contiendra  donc des mthodes de classe servant  lire certains types d'objets
// comme les chanes de caractres
 */

package entreesSorties;

import java.awt.image.BufferedImage;
import java.io.*;
import java.util.*;

import javax.imageio.ImageIO;

import objets.CElementGenerique;
import objets.CListeObjets;
import objets.CObjetBase;
import types.Int;


// Lorsque le C++ enregistre des objets dans un flux, il maitient un tableau
// qui est incrment  chque chargement d'un nouvel objet et chaque lment de ce
// tableau d'indice i est une chaine contenant le nom de la i - 1 ime classe 
// Curieusement lorsqu'une classe est rencontre pour la premire fois, son nom est
// enregistr deux fois dans la liste
public class MathGraphDataInputStream extends DataInputStream {
  // Version 2.5 C++ : Dfinition du n de version du logiciel.
	// public static int numeroVersionActif = 5;
	// Version Java 2.0 : Changement de numro de version pour deux modes d'affichage
	// public static int numeroVersionActif = 6;
	// Version Java 3.0 : Changement de numro de version car dans OperateurFonction les indices
	// des fonctions complexes ont t dcals
	// Version Java 4.4 : On change de numro de version car l'ventuelle image de fond est maintenant
	// directement enregistre dans le flus de donnes
	// public static int numeroVersionActif = 7;
	// public static int numeroVersionActif = 8;	
	// Version 4.5 : On passe  un nouveau numro de version car les objets hritant de CAffichageLieAPoint
	// prennent deux nouveaux paramtres pour l'alignement horizontal ou vertical.
	// public static int numeroVersionActif = 9;
	// Version 4.6 : On passe  un nouveau numro de version car on spcifie dans le document si les champs d'dition
	// doivent avoir des boutons Valeurs et fonctions prdfinies associs.
	// Version 4.7.7 : On passe au numro de version 11 car si la figure contient une image de fond, la version JavaScript
	// a besoin ue ses dimensions soient insres dans le flux
	// public static int numeroVersionActif = 10;
	public static int numeroVersionActif = 11;
  // Tableau de bytes pour la conversion de double big endian en little endian
  // Erreur corrige pour la version 1.9.5
  // Ce tableau tait dclar comme static ce qui faisait des erreurs alatoires de chargement
  // lorsque plusieurs applets taient insres sur une mme page
  byte[] bufferd = new byte[8];
  // Pour pouvoir charger les id de menus qui taient chargs comme des objets
  // il faut pouvoir accder  ces deux membres dans CMathGraphDoc.chargeDepuisFlx
  public ArrayList<String> listeNomsClasse = new ArrayList<String>();
  public ArrayList<Int> listeNumerosVersion = new ArrayList<Int>();
  private int indiceNature; // Contient la nature de l'objet charg dans le cas d'un lment gnrique

  /** Creates new MathGraphDataInputStream */
  public MathGraphDataInputStream(InputStream in) {
    super(in);
    }
  // Renvoie un entier sur deux octets. Les octets ont t enregistrs dans l'ordre inverse par rapport
  // au langage Java
  final public int readCShort() throws IOException {
    int b1 = (int) readUnsignedByte();
    int b2 = (int) readUnsignedByte();
    return (int)(((b2 << 8) & 0xFF00) | b1);
  }
  
  // Idem pour un entier sur quatre octets
  final public int readCInt() throws IOException {
    int b1 = (int) readUnsignedByte();
    int b2 = (int) readUnsignedByte();
    int b3 = (int) readUnsignedByte();
    int b4 = (int) readUnsignedByte();
    return (int)(((b4 << 24) & 0xFF000000) |((b3 << 16) & 0x00FF0000) | ((b2 << 8) & 0x0000FF00) | b1);
  }
  
  // Lecture d'un double little endian
  final public double readCDouble() throws IOException {
    bufferd[0] = (byte) readUnsignedByte();
    bufferd[1] = (byte) readUnsignedByte();
    bufferd[2] = (byte) readUnsignedByte();
    bufferd[3] = (byte) readUnsignedByte();
    bufferd[4] = (byte) readUnsignedByte();
    bufferd[5] = (byte) readUnsignedByte();
    bufferd[6] = (byte) readUnsignedByte();
    bufferd[7] = (byte) readUnsignedByte();
    long longBits = 0;
    for (int i = 7; i >= 0; i--) {
     long b = bufferd[i];
     b &= 0xff;
     longBits |= (b << (i * 8));
    } 
    return Double.longBitsToDouble(longBits);
  }

  private final int readCStringLength() throws IOException {
    int nNewLen;

    // attempt BYTE length first
    int bLen = readUnsignedByte();

    if (bLen != 0xff)
      return bLen;

    // attempt WORD length
    short wLen = (short) readCShort();
    if (wLen == 0xfffe)
    {
      // UNICODE string prefix (length will follow)
      return -1;
    }
    else if (wLen == 0xffff)
    {
      // read DWORD of length
      nNewLen = readCInt();
      return nNewLen;
    }
    else
      return wLen;
  }
  
  // Renvoie une chane qui a t charge en C++
  // Les chaines CString en C++ sont enrgistres dans les flux de la faon suivante : si la chane fait
  // moins de 254 octets de long, un octet pour la longueur de la chane suivi des caractres.
  // Si la chane fait plus, un octet FF de 255, suivi de deux octets reprsentant la longueur de la chane
  // suivis des caractres
   
  public final String readCString() throws IOException {
    int longueur = readCStringLength();
    byte[] tab = new byte[longueur];
    for (int i = 0; i < longueur; i++) {
      tab[i] = readByte();
    }
    return new String(tab,"ISO-8859-1");
  }
  
  // Idem pour lire une chane dont on connat dj la longueur
  public final String readCString(int longChaine) throws IOException {
    byte[] tab = new byte[longChaine];
    for (int i = 0; i < longChaine; i++) {
      tab[i] = readByte();
    }
    return new String(tab,"ISO-8859-1");
  }
  

  // Modification pour la version 1.9.5. Filtrage des caractres grecs
  // et des caractres spciaux de la chaine charge
  public final String readCStringAvecFiltrage() throws IOException {
    String ch = readCString();
    String ch1 = glob.Kernel.chaineAvecCaracteresGrecsUnicode(ch);
    return glob.Kernel.chaineAvecCaracteresSpeciauxUnicode(ch1);
  }
  
    
  
  // Lit un objet depuis le flux. Deux possibilits : soit c'est la premire fois qu'un objet de
  // ce type est lu. Alors Les deux premiers octets sont 0xffff, les deux octets suivants contienne
  // le numro de version de l'objet, les deux suivants contiennent la longueur de la chaine
  // contenant le nom de la classe suivi du nom de la classe puis viennent les donnes
  // de la classe srializes dans le flux.
  // Soit ce n'est pas la premire fois qu'un tel objet est charg et les deux premiers octets
  // contiennet un code spcifiquue au C++ pour cette classe. Suivent alors les donnes de la classe srialize
  // dans le flux.
  @SuppressWarnings({ "rawtypes" })
	public final CObjetBase readCObject(CListeObjets liste)
    throws IOException {
    Class classe;
    CObjetBase ob;
    Int numVersion = new Int();
    String nomClasse = readNomClasse(numVersion, liste.numeroVersion);  
    try {
      classe = Class.forName("objets." + nomClasse);
    }
    catch (ClassNotFoundException e) {
      throw new IOException();      
    }
    try {
      ob = (CObjetBase) classe.newInstance();
      ob.nVersion = numVersion.getValue();
      // Un objet gnrique a besoin de connatre la nature de l'objet qu'il reprsente
      if (classe == CElementGenerique.class) 
      	((CElementGenerique) ob).adapteNature(indiceNature);
    }
    catch (InstantiationException e) {
      throw new IOException();      
    }      
    catch (IllegalAccessException e) {
      throw new IOException();      
    }      
    ob.chargeDepuisFlux(this, liste);
    return ob;   
  }
  
  public final String readNomClasse(Int numVer, int numeroVersionObjet) throws IOException {
    String nomClasse;
    int s = readCShort();
    if (s == 0Xffff) {
      numVer.setValue(readCShort());
      Int numeroVer = new Int(numVer.getValue()); // C'est un nouvel objet qui doit tre
      // rajout  la liste des numros de version des objets srializs
      int longueurNomClasse = (int) readCShort();
      nomClasse = readCString(longueurNomClasse);
      listeNomsClasse.add(nomClasse);
      listeNumerosVersion.add(numeroVer);
      listeNomsClasse.add(nomClasse);
      listeNumerosVersion.add(numeroVer);
    }
    else {
      nomClasse = (String)(listeNomsClasse.get(s - 32769));
      numVer.setValue(listeNumerosVersion.get(s - 32769).getValue());
      Int numeroVer = new Int(numVer.getValue()); // C'est un nouvel objet qui doit tre
      listeNomsClasse.add(nomClasse);
      listeNumerosVersion.add(numeroVer);
    }
    if (nomClasse.equals("CAction")) {
      // Le type CAction n'est plus utilis
      // On charge la nature de l'objet qui n'est plus utilise car rendu par getNature
      // de CElementBase
      @SuppressWarnings("unused")
      // Pour la version 1 du logiciel la nature tait stocke en valeur relle en 32 bits
      // Pour les version suivantes un byte contenait l'indice de la nature de l'objet
      int k;
			byte b;
			if (numVer.getValue() == 1) k = readCInt();
			else {
				b = readByte();
				indiceNature = b;
			}
      return readNomClasse(numVer, numVer.getValue());
    }
    else return nomClasse;
  }

   // Fonction renvoyant lengthnombre d'objets d'une liste d'objets srialize
  // dans un flux C++
  public final int readCCount() throws IOException {
    int wCount = readCShort();
    if (wCount != 0xFFFF)
      return (int) wCount;
    int dwCount = readCInt();
    return dwCount;
  }
  
  /* Nouvelle version de serialization en java. Lecture */
  @SuppressWarnings("rawtypes")
	public final CObjetBase readObject(CListeObjets liste)
    throws IOException {
    Class classe;
    CObjetBase ob;
    String nomClasse;
    int numVer; // Numro de version
    // On lit d'abord le nom de classe de l'objet  charger
    // Pour cela on lit d'abord les deux premiers octets. S'il valent FFFF c'est la premire fois
    // qu'on charge un objet de ce type. Sinon il reprsente l'indice dans les listes
    // associes de la classe d'objet qu'on est en train de charger
    int s = readInt();
    if (s == -1) {
      numVer = readInt();
      Int numeroVer = new Int(numVer); // C'est un nouvel objet qui doit tre
      // rajout  la liste des numros de version des objets srializs
      nomClasse = readUTF();
      listeNomsClasse.add(nomClasse);
      listeNumerosVersion.add(numeroVer);
    }
    else {
      nomClasse = (String)(listeNomsClasse.get(s));
      numVer = listeNumerosVersion.get(s).getValue();
    }
    try {
      classe = Class.forName(nomClasse);
    }
    catch (ClassNotFoundException e) {
      throw new IOException();      
    }
    try {
      ob = (CObjetBase) classe.newInstance();
      ob.nVersion = numVer;
    }
    catch (InstantiationException e) {
      throw new IOException();      
    }      
    catch (IllegalAccessException e) {
      throw new IOException();      
    }      
    ob.read(this, liste);
    return ob;   
  }
  
  
  /**
   * Ajout version 4.4.
   * On lit une bufferedImage depuis le flux
   * On charge d'abord le nombre total de bytes enregistrs pour reprsenter cette image
   * puis le byte array reprsentant l'image
   * @return
   * @throws IOException
   */
  public final BufferedImage readBufferedImage() throws IOException {
 	    int size = readInt();
	    byte[] b = new byte[size];
	    readFully(b);
	    ByteArrayInputStream bais = new ByteArrayInputStream(b);
	  	BufferedImage im = ImageIO.read(bais);
	  	return im;
  }
 
}
