/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.*;
import java.io.*;
import java.util.ListIterator;
import java.util.StringTokenizer;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

public class CCommentaire extends CAffichageLieAPoint {
	public String chaineCommentaire;
	// Nouveau pour la version 1.8.
	// Il faut maintenir une liste dont les lments pointent sur les valeurs
	// affiches par ce commentaire
	public CListeObjets listeValeursDynamiquesUtilisees;
  // La chaine d'affichage est la chane relle  afficher pes qu'on ait regard s'il y a
  // des valeurs dynamiques dans lengthcommentaire
  protected String chaineAffichage;
  /** Creates new CCommentaire */
  public CCommentaire() {
  }
  
  // Version 4.6.4. On change le numro de version et si le numro de version est infrieur  2 on remplace les
  // codes #V, #R, #A, #H et #L par des codes LaTeX quivalents
  @Override
  public int numeroVersion() {
  	return 2;
  }  

  // Constructeur pour clonage
  public CCommentaire(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
      CPointAncetre pointLieInit, int indiceTaillePoliceInit, int encadrementInit,
      boolean effacementFondInit, Color couleurFondInit, String chaineCommentaireInit,
      	int alignementHorizontalInit, int alignementVerticalInit) {
        super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
          dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit, encadrementInit, effacementFondInit, couleurFondInit,
          alignementHorizontalInit, alignementVerticalInit);
        chaineCommentaire = chaineCommentaireInit;
        listeValeursDynamiquesUtilisees = new CListeObjets(liste.uniteAngle, liste.pointeurLongueurUnite);        
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    // Corrig lors de la version 2.0
    int ind1 = listeSource.indexOf(pointLie);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CCommentaire(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y,
      masque, (CPointAncetre)listeCible.get(ind1, CPointAncetre.class), indiceTaillePolice, encadrement,
      effacementFond, couleurFondAffichage, chaineCommentaire, alignementHorizontal, alignementVertical);
    ((CCommentaire) ptelb).listeValeursDynamiquesUtilisees = 
      new CListeObjets(listeCible.uniteAngle, listeCible.pointeurLongueurUnite);
    if (listeCible.getClass() != CPrototype.class) ((CCommentaire) ptelb).determineDependances();
    return ptelb;
  }
  
  // Rajout pour version 2.0. Un commentaire est utilis pour les affichages fugitifs
  // au voisinage du pointeur de souris.
  // Fonction servant  changer la chane affiche par le commentaire
  public void donneChaine(String chaine) {
    chaineCommentaire = chaine;
  }
  
  // Rajout pour la version 2.0 pour la mme raison que plus haut.
  // Modifi verson 4.5.2
  public void placeEn(int x, int y) {
    x_nom = x;
    y_nom = y;
  }
    
  // Inutile de redfinir efface()
  public String rendChaineAffichage(boolean infoRandom) {
  	return chaineAffichage;
  }
 
  @Override
  public String infoPourHistorique() {
    String ch;
    if (pointLie == null) ch = getString("chinfo47") + " " + getString("chinfo48");
    else ch = getString("chinfo47") + " " + getString("chinfo49");
    return ch;
  }
  
  @Override
  public void ajouteAntecedents(CListeObjets liste) {
    if (pointLie != null) liste.add(pointLie);
  } 
  
  /**
   * Nouveau pour version 1.9 pour l'affichage de formule (sert pour l'applet)
   * Renvoie true dans formule s'il s'agit d'un affichage de formule et false sinon
   */
  int indicePremierValOuForm(int indDeb, Bool formule)
  {
    int indVal = chaineCommentaire.indexOf("#Val(", indDeb);
    int indFor = chaineCommentaire.indexOf("#For(", indDeb);
    if ((indVal == -1) && (indFor == -1)) return -1;
    if (((indVal > indFor) || (indVal == -1)) && (indFor != -1))
    {
      formule.setValue(true);
      return indFor;
    }
    else
    {
      formule.setValue(false);
      return indVal;
    }
  }
  
  /**
   * Doit tre appel lors de la srialization (chargement), lors de la cration d'un
   * commentaire ou de sa modification.
   * Est similaire  Positionne() et doit le rester
   */
  public void determineDependances()
  {
    Bool bformule = new Bool(true);
    listeValeursDynamiquesUtilisees.retireTout();
    // On examine la chaine  la recherche de #Val(
    int len = chaineCommentaire.length();
    int jdeb;
    int j = 0;
    if ((chaineCommentaire.indexOf("#Val(",j) == -1) 
      && (chaineCommentaire.indexOf("#For(",j) == -1))
    {
      return;
    }
    while (((jdeb = indicePremierValOuForm(j, bformule)) != -1) && j < len)
    {
      // On recherche la parenthse fermante correspondante
      int indparf = chaineCommentaire.indexOf(')',jdeb + 5);
      if (indparf == -1)
      {
        j = len + 1;
      }
      else
      {
        // On recherche aussi un ventuel retour  la ligne
        int indsaut = chaineCommentaire.indexOf('\n',jdeb + 5);
        if ((indsaut !=  -1) && (indsaut < indparf))
        {
          j = indsaut + 1;
        }
        else
        {
          // On cre une chaine form de ce qu'il y a entre #Val( et la parenthse fermante
          String ch = chaineCommentaire.substring(jdeb+5, indparf);
          // On retire les ventuels espaces de cete chaine
          int k;
          while((k = ch.indexOf(' ', 0)) != - 1)
          {
            ch = ch.substring(0,k) + ch.substring(k+1);
          }
          // On spare cette chaine avec les virgules
          StringTokenizer st = new StringTokenizer(ch, ",",false);
          if (! st.hasMoreTokens())
          {
            j = indparf + 1;
          }
          else
          {
            String nomvaleur = st.nextToken();
            // On recherche si c'est bien le nom d'une valeur valide
            // Il ne faut pas que la valeur ait t dfinie aprs l'affichage de valeur
            int indicecommentaire = listeProprietaire.indexOf(this);
            // Trs important : quand determineDependances est appel
            // par getClone() de CCommentaire, le commentaire n'a pas encor t rajout
            //  la liste clone et ainsi indiceCommentaire renvoie -1.
            // dans ce cas, indiceCommentaire doit valoir le nombre d'lments actuels
            // de la liste clone qui le possde
            if (indicecommentaire == -1) indicecommentaire = listeProprietaire.size() - 1;
            CValeurDynamique pValeur = pointeur(NatureCalcul.NTouteValeurPourCommentaireDynamique, nomvaleur,indicecommentaire);
            if (pValeur == null)
            {
              j = indparf + 1;
            }
            else
            {
            	long natc = pValeur.getNatureCalcul();
              if ((bformule.getValue() && ((natc & (NatureCalcul.NCalculReelOuComplexe | NatureCalcul.NTouteFonctionReelleOuComplexe))!= 0))
                  || !bformule.getValue()) {
                listeValeursDynamiquesUtilisees.add(pValeur);
              }
              // Si la valeur n'existe pas, on ne l'inclut pas dans la chane  afficher
              j = indparf + 1;
            }
          }
        }
      }		
    }
  } 
  
  // Pour la version 1.8, positionne() est redfini
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    Bool bformule = new Bool(true);
    // On examine la chaine  la recherche de #Val(
    int len = chaineCommentaire.length();
    StringBuffer bufferAffichage = new StringBuffer(len+128);
    int jdeb;
    int j = 0;
    if ((chaineCommentaire.indexOf("#Val(",j) == -1) 
      && (chaineCommentaire.indexOf("#For(",j) == -1)) {
        chaineAffichage = chaineCommentaire;
        super.positionne(infoRandom, dimfen);
        return;
    }
    while (((jdeb = indicePremierValOuForm(j, bformule)) != -1) && j < len)
    {
      bufferAffichage.append(chaineCommentaire.substring(j, jdeb));
      // On recherche la parenthse fermante correspondante
      int indparf = chaineCommentaire.indexOf(")",jdeb + 5);
      if (indparf == -1)
      {
        bufferAffichage.append(chaineCommentaire.substring(jdeb, len));
        j = len + 1;
      }
      else
      {
        // On recherche aussi un ventuel retour  la ligne
        int indsaut = chaineCommentaire.indexOf('\n',jdeb + 5);
        if ((indsaut !=  -1) && (indsaut < indparf))
        {
          bufferAffichage.append(chaineCommentaire.substring(jdeb, indsaut+2));
          j = indsaut + 1;
        }
        else
        {
          // On cre une chaine form de ce qu'il y a entre #Val( et la parenthse fermante
          String ch = chaineCommentaire.substring(jdeb+5, indparf);
          // On retire les ventuels espaces de cete chaine
          int k;
          while((k = ch.indexOf(' ', 0)) != - 1)
          {
            ch = ch.substring(0, k) + ch.substring(k+1);
          }
          // On spare cette chaine avec les virgules
          StringTokenizer st = new StringTokenizer(ch, ",",false);
          if (! st.hasMoreTokens())
          {
            bufferAffichage.append(chaineCommentaire.substring(jdeb, indparf+1));
            j = indparf + 1;
          }
          else
          {
            String nomvaleur = st.nextToken();
            // On recherche si c'est bien le nom d'une valeur valide
            // Il ne faut pas que la valeur ait t dfinie aprs l'affichage de valeur
            // Si c'est le nom d'une valeur utilise, elle est comprise dans la
            // liste listeValeursDynamiquesUtilisees
            int indfin = listeValeursDynamiquesUtilisees.size() - 1;
            // Modifi version 4.7.2
            // CValeurDynamique pValeur = listeValeursDynamiquesUtilisees.pointeurValeurOuFonction(nomvaleur,indfin);
            CValeurDynamique pValeur = listeValeursDynamiquesUtilisees.pointeurParNatureCalcul
            		(NatureCalcul.NTouteValeurPourCommentaireDynamique, nomvaleur,indfin);
            if (pValeur == null)
            {
              bufferAffichage.append(chaineCommentaire.substring(jdeb, indparf+1));
              j = indparf + 1;
            }
            else
            {
              
              // Si la valeur n'existe pas, on ne l'inclut pas dans la chane  afficher
              if (pValeur.existe)
              {
                if (bformule.getValue()) {
                // L'affichage de formule n'a de sens que pour un calcul ou une fonction
                	long natc = pValeur.getNatureCalcul();
                  if ((natc & (NatureCalcul.NCalculReelOuComplexe | NatureCalcul.NTouteFonctionReelleOuComplexe)) != 0)
                    bufferAffichage.append (((CCalcul)pValeur).chaineCalcul);
                }
                else {
                  String chnombre = new String("");
                  if (st.hasMoreTokens())
                  {
                    String chnbdec = st.nextToken();
                    if (chnbdec.equals("+")) {
                      chnombre = pValeur.rendChaineValeurPourCommentaire(2, infoRandom);
                      if (!chnombre.startsWith("-")) chnombre = "+ " + chnombre;
                    }
                    else {
                      Integer nbdec = new Integer(chnbdec);
                      int nbdecimales = nbdec.intValue();
                      chnombre = pValeur.rendChaineValeurPourCommentaire(nbdecimales, infoRandom);
                      if (st.hasMoreTokens())
                      {
                        String chplus = st.nextToken();
                        if (chplus.equals("+"))
                        {
                        	if (!chnombre.startsWith("-")) chnombre = "+ " + chnombre;
                        }
                      }
                    }
                  }
                  else
                  {
                    // Si le nombre de dcimales n'est pas prcis, on en met deux par dfaut
                    chnombre = pValeur.rendChaineValeurPourCommentaire(2, infoRandom);
                  }
                  bufferAffichage.append(chnombre);
                }
              }
              j = indparf + 1;
            }
          }
        }
      }		
    }
    if (j < len) bufferAffichage.append(chaineCommentaire.substring(j, len));
    chaineAffichage = bufferAffichage.toString();
    super.positionne(infoRandom, dimfen);
  }
  
  /**
   * Appel par CLatex dans le cas d'uen erreur Latex. dans ce cas l'affichae est remplac par un message d'erreur
   * @param infoRandom
   * @param dimfen
   */
  public void positionnePourCLatex(boolean infoRandom, DimensionFenetre dimfen) {
  	super.positionne(infoRandom, dimfen);
  }
 
  @Override
  public boolean dependDe(CElementBase p) {  
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || listeValeursDynamiquesUtilisees.dependDe(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || super.dependDePourBoucle(p)
    || listeValeursDynamiquesUtilisees.dependDePourBoucle(p);
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NCommentaire;
  }
  
  
  @Override
  public String informationModification() {
  	return getString("chaineOutil55");
  }
  
	public final CValeurDynamique pointeur(long nat, String nom, int indiceMaxi) {
		int i;
		CElementBase elb;
		// Si le CLatex est un objet final de construction on recherhe d'abord la valeur
		// dans les objets intermdiaires ou finaux de cette construction
		// Important : on ne peut pas utiliser impProto car lors de getClone appel par CImplementationProto.implemente
		// le membre impProto n'a pas encore t tabli
		// Modifi version 4.5 : Aussi  faire pour les objets intermdiaires car un CLatex ou un CCommentaire peut gnrer un lieu d'objets.
		// if (estElementFinal) {
		if (estElementFinal || estElementIntermediaire()) {
			i = indiceMaxi;
			while (i >= 0) {
				elb = listeProprietaire.get(i);
				if (elb.estElementFinal || elb.estElementIntermediaire()) {
					if (elb.estDeNatureCalcul(nat)) {
						if (((CValeurDynamique) elb).nom().equals(nom)) {
							return (CValeurDynamique) elb;
						} else {
							i--;
						}
					} else {
						i--;
					}
				}
				else break;
			}			
		}
		// Sinon on recherche dans tous les objets qui ne sont pas de objets intermdiaires de construction
		i = 0;
		ListIterator<CElementBase> li = listeProprietaire.listIterator();
		while (i <= indiceMaxi) {
			elb = li.next();
			if (! elb.estElementIntermediaire() && elb.estDeNatureCalcul(nat)) {
				if (((CValeurDynamique) elb).nom().equals(nom)) {
					return (CValeurDynamique) elb;
				} else {
					i++;
				}
			} else {
				i++;
			}
		}
		return null;
	}
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    // Modification pour la version 1.9.5 : filtarge des caractres grecs
    chaineCommentaire = Kernel.sansCR(inps.readCStringAvecFiltrage());
    y_nom += Kernel.decalagePourCompatibilite(chaineCommentaire);
    remplaceAnciensCodesParLatex();
    listeValeursDynamiquesUtilisees = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);        
    if (list.getClass() != CPrototype.class) determineDependances();
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    chaineCommentaire = inps.readUTF();
    // Nouveau verion 4.6.4 : on filtre es anciens codes pour les remplacer par du LaTeX
    if (nVersion < 2) remplaceAnciensCodesParLatex();
    listeValeursDynamiquesUtilisees = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);        
    if (listeProprietaire.getClass() != CPrototype.class) determineDependances();
  }
  
  /**
   * Rajout version 4.6.4.
   * SDoit tre appel par CLatex.read
   */
  public void readForCLatex(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    chaineCommentaire = inps.readUTF();
    listeValeursDynamiquesUtilisees = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);        
    if (listeProprietaire.getClass() != CPrototype.class) determineDependances();
  }
  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeUTF(chaineCommentaire);
  }

  @Override
  public String chaineDesignation() {
    return getString("designationComentaire");
  }
  
  
  private void remplaceAnciensCodesParLatex() {
  	try {
  		remplaceAnciensCode("#V(", "\\overrightarrow");
  		remplaceAnciensCode("#A(", "\\overline");
  		remplaceAnciensCode("#R(", "\\sqrt");
  		remplaceAnciensCode("#H(", "^");
  		remplaceAnciensCode("#L(", "_");
  	}
  	catch (Exception e) {
		}
  }
  
  private void remplaceAnciensCode(String ancienCode, String codeLatex) {
  	StringBuffer stb = new StringBuffer(chaineCommentaire);
  	int ind = 0;;
  	while ((ind = stb.indexOf(ancienCode, ind)) != -1) {
  		// On recherche la parenthse fermant correspondante
  		int indparfer = parentheseFermante(stb.toString(), ind+3);
  		if (indparfer == -1) ind = ind + 3;
  		else {
	  		String ch = stb.substring(ind + 3, indparfer);
	  		stb.delete(ind, indparfer+1);
	  		stb.insert(ind, "$"+codeLatex + "{"+ ch + "}$");
  		}
  	}
  	chaineCommentaire = stb.toString();
  }
  
	private static final int parentheseFermante(String chaine, int pdebut) {
		int p;
		char ch;
		int somme;

		somme = 1;
		p = pdebut;
		int len = chaine.length();
		while ((somme  != 0) && (p <len))  {
			ch = chaine.charAt(p);
			if (ch == '(')
				somme++;
			else {
				if (ch == ')')
					somme--;
			}
			p++;
		}
		if (somme == 0) return p - 1; else return -1;
	}

}