/*
 * CDemiPlan.java
 *
 * Created on 20 juillet 2004, 18:39
 */

/**
 *
 * @author  Yves
 * @version 
 */
package objets;

import java.io.*;
import java.awt.*;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CDemiPlan extends CSurface {
	private int[] abscissesPointsBordure;
	private int[] ordonneesPointsBordure;
	byte nombrePointsBordure; // Nombre de sommets de ce polygne
	// rellement utiliss.
	// Une quation cartsienne du bord est ax+by+c = 0
	double a, b, c;
	double signePointDemiPlan; // Vaudra 1 si le point PointDemiPlan
	// est dans le demi-plan, - 1 s'il n'y est pas et 0 s'il est sur
	// la droite
	// Point dfinissant lequel des deux demi-plans est le bon
	CPointAncetre pointDemiPlan;

  /** Creates new CDemiPlan */
  public CDemiPlan() {
    abscissesPointsBordure = new int[5];
    ordonneesPointsBordure = new int[5];
  }
  
  // Constructeur pour clonage
  public CDemiPlan(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleRemplissageInit, CDroiteAncetre bordInit, CPointAncetre pointDemiPlanInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleRemplissageInit, bordInit);
    pointDemiPlan = pointDemiPlanInit;
    abscissesPointsBordure = new int[5];
    ordonneesPointsBordure = new int[5];
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(bord);
    int ind2 = listeSource.indexOf(pointDemiPlan);
    int ind3 = listeSource.indexOf(impProto);       
    CElementBase ptelb = new CDemiPlan(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, masque, styleRemplissage, (CDroiteAncetre)(listeCible.get(ind1, CDroiteAncetre.class)),
        (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)));
    return ptelb;
  }

  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    nombrePointsBordure = ((CDemiPlan)ptel).nombrePointsBordure;
    for (int i = 0; i < nombrePointsBordure; i ++) {
      abscissesPointsBordure[i] = ((CDemiPlan) ptel).abscissesPointsBordure[i];
      ordonneesPointsBordure[i] = ((CDemiPlan) ptel).ordonneesPointsBordure[i];
    }
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
  	boolean res = super.confonduAvec(p);
   	if (!res) return false;
  	CDemiPlan pt = (CDemiPlan) p;
  	res = res && (pointDemiPlan == pt.pointDemiPlan);
  	return res;
  }
   
  // fonction appele seulement dans Positionne;
  boolean dansDemiPlan(double x, double y)
  {
    return ((a*x+b*y+c)*signePointDemiPlan >= 0);
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || pointDemiPlan.dependDe(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return super.dependDePourBoucle(p) || pointDemiPlan.dependDePourBoucle(p);
  }
  
  // Oubli jusqu' la version 2.5.3
  public boolean dependDePourCapture(CElementBase p) {
    return super.dependDePourCapture(p) || pointDemiPlan.dependDePourCapture(p); 	
  }
  

  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    CDroite droiteBord = (CDroite) bord;
    int xext1 = (int) Math.round(droiteBord.xext1);
    int xext2 = (int) Math.round(droiteBord.xext2);
    int yext1 = (int) Math.round(droiteBord.yext1);
    int yext2 = (int) Math.round(droiteBord.yext2);
    existe = (pointDemiPlan.existe && droiteBord.existe) && (!pointDemiPlan.surDroite(droiteBord));
    if (!existe) return;
    // Une quation cartsienne du bord est ax+by+c = 0.
    b = -droiteBord.vecteur.x;
    a = droiteBord.vecteur.y;
    c = - a* xext1 - b*yext1;
    double s = a*pointDemiPlan.x+b*pointDemiPlan.y+c;
    if (s > 0)
    {
      signePointDemiPlan = 1;
    }
    else
    {
      if (s < 0) signePointDemiPlan = -1;
      else signePointDemiPlan = 0;
    }
    // Les coordonnes des 4 coins de la fentre
    double[] x = new double[4];
    double[] y = new double[4];
    x[0] = 0;
    y[0] = 0;
    x[1] = dimfen.longueurAbscisses;
    y[1] = 0;
    x[2] = dimfen.longueurAbscisses;
    y[2] = dimfen.longueurOrdonnees;
    x[3] = 0;
    y[3] = dimfen.longueurOrdonnees;
    // Les rsultats pour savoir s'ils sont ou non dans le demi-plan
    boolean[] dedans = new boolean[4];
    int i;
    int j = -1;
    for (i = 0; i <= 3; i ++)
    {
      dedans[i] = dansDemiPlan(x[i], y [i]);
      if ((j == -1) && dedans[i]) j = i;
    }
    if (j == -1) 
    {
      existe = false;
      return; // Aucun sommet dans le demi-plan
    }
    nombrePointsBordure = 0;
    for (i = 0; i <= 3; i ++)
    {
      j = (j + 1) % 4;
      if (dedans[j])
      {
        abscissesPointsBordure[nombrePointsBordure] = (int) Math.round(x[j]);
        ordonneesPointsBordure[nombrePointsBordure] = (int) Math.round(y[j]);
        nombrePointsBordure++;
      }
      else
      {
        if ((j % 2) == 0)
        {
          if (xext1 == x[j])
          {
            abscissesPointsBordure[nombrePointsBordure] = (int) Math.round(xext1);
            ordonneesPointsBordure[nombrePointsBordure] = (int) Math.round(yext1);
            nombrePointsBordure++;
            abscissesPointsBordure[nombrePointsBordure] = (int) Math.round(xext2);
            ordonneesPointsBordure[nombrePointsBordure] = (int) Math.round(yext2);
            nombrePointsBordure++;
          }
          else
          {
            abscissesPointsBordure[nombrePointsBordure] = (int) Math.round(xext2);
            ordonneesPointsBordure[nombrePointsBordure] = (int) Math.round(yext2);
            nombrePointsBordure++;
            abscissesPointsBordure[nombrePointsBordure] = (int) Math.round(xext1);
            ordonneesPointsBordure[nombrePointsBordure] = (int) Math.round(yext1);
            nombrePointsBordure++;
          }
        }
        else
        {
          if ( yext1 == y[j])
          {
            abscissesPointsBordure[nombrePointsBordure] = (int) Math.round(xext1);
            ordonneesPointsBordure[nombrePointsBordure] = (int) Math.round(yext1);
            nombrePointsBordure++;
            abscissesPointsBordure[nombrePointsBordure] = (int) Math.round(xext2);
            ordonneesPointsBordure[nombrePointsBordure] = (int) Math.round(yext2);
            nombrePointsBordure++;
          }
          else
          {
            abscissesPointsBordure[nombrePointsBordure] = (int) Math.round(xext2);
            ordonneesPointsBordure[nombrePointsBordure] = (int) Math.round(yext2);
            nombrePointsBordure++;
            abscissesPointsBordure[nombrePointsBordure] = (int) Math.round(xext1);
            ordonneesPointsBordure[nombrePointsBordure] = (int) Math.round(yext1);
            nombrePointsBordure++;
          }
        }
        while (i <= 3)
        {
          if (dedans[j])
          {
            abscissesPointsBordure[nombrePointsBordure] = (int) Math.round(x[j]);
            ordonneesPointsBordure[nombrePointsBordure] = (int) Math.round(y[j]);
            nombrePointsBordure++;
          }
          i++;
          j = (j + 1) % 4;
        }
      }
    }
  }

  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    int styleRemplissageEffectif;
    if (masque) styleRemplissageEffectif = StyleRemplissage.QuadrillageCroise;
    else styleRemplissageEffectif = styleRemplissage;
    if (styleRemplissageEffectif == StyleRemplissage.QuadrillagePleinTransp) {
    	if (transparence != 1) {
    		pDC.setColor(couleur);
  	    Composite originalComposite = pDC.getComposite();
  	    pDC.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, transparence));
		    pDC.fillPolygon(abscissesPointsBordure, ordonneesPointsBordure, nombrePointsBordure);
  	    pDC.setComposite(originalComposite);    		
    	}
    	else {
    		pDC.setColor(couleur);
		    pDC.fillPolygon(abscissesPointsBordure, ordonneesPointsBordure, nombrePointsBordure);	
    	}
    }
    else {
      if (styleRemplissageEffectif == StyleRemplissage.QuadrillagePlein) {
    		pDC.setColor(couleur);
		    pDC.fillPolygon(abscissesPointsBordure, ordonneesPointsBordure, nombrePointsBordure);	
    	}
      else {
	    	TexturePaint tp = StyleRemplissage.getTexturePaint(couleur, styleRemplissageEffectif, listeProprietaire.basseDefinition, couleurFond);
	    	pDC.setPaint(tp);
			  pDC.fillPolygon(abscissesPointsBordure, ordonneesPointsBordure, nombrePointsBordure);	
      }
    }
  }    
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (pointDemiPlan== ancienPoint) pointDemiPlan = nouveauPoint;
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NDemiPlan;
  }
  
  public CDroite droiteBord() {
  	return (CDroite) bord;
  }
  
  public double distancePoint(double xp, double yp, boolean masquage) {
  	 if (!(existe) || (masquage && masque)) return -1;
  	 else return droiteBord().distancePointPourDemiPlan(xp,yp);
  }

  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
   super.chargeDepuisFlux(inps, list);
   int ind1 = inps.readCInt();
   pointDemiPlan = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
  }
  /* Enregistrement dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    pointDemiPlan = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
  }  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(pointDemiPlan);
    oups.writeInt(ind1);
  }     
}
