/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 *
 * @author  Yves
 * @version
 */
package objets;

import java.util.*;
import java.awt.*;
import java.io.*;
import java.awt.geom.Point2D;

import dialogs.EditeurNomCalcul;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

@SuppressWarnings("serial")
public class CListeObjets extends ArrayList<CElementBase> {
	// Nouveaut pour la version 2.5 : Gestion d'un numro de version qui est
	// celui
	// du document qui a charg la liste lors d'un chargement et puis ensuite est
	// le numro
	// de version actuel du logiciel C++
	public int numeroVersion;
	// Dans certains cas on aura besoin de savoir quel est le document
	// propritaire de la liste,
	// en particulier lors du chargement des macros sonores et des images
	// Ce pointeur sera null par dfaut
	public CMathGrafDoc documentProprietaire;
	public int uniteAngle;
	public CLongueur pointeurLongueurUnite;
	public CMacro macroParDefaut; // Pointeur sur l'ventuelle
	// Distance mini pour qu'un objet soit considr comme proche du pointeur
	// souris
	public static double distanceMiniPourEtreProche = 4;
	// Modification  partir de la version java : uand on imprime
	// la figure il faut passer dans un systme de coordonnes logiques qui est
	// diffrent du systme de coordonnes cran
	// Chaque CListeObjets instanciera uun boolean basseDefinition qui contiendra
	// true si on est en rsolution
	// cran et false sinon
	public boolean basseDefinition;
	// Ajout version 3.2.8 pour lutter contre les dpassements de capacit dans les sommes indices
	// et les calculs d'intgrales
	// Initialis  zro avant tout calcul par initialisenombreIterations()
	public long nombreIterations;

	// Macro  excuter avant copiage ou impression. null par dfaut
	/** Creates new CListeObjets */
	public CListeObjets() {
		super(20);
		numeroVersion = MathGraphDataInputStream.numeroVersionActif;
		documentProprietaire = null;
		basseDefinition = true;
		pointeurLongueurUnite = null;
	}

	public CListeObjets(int uniteAngleInit, CLongueur pointeurLongueurUniteInit) {
		super(20);
		numeroVersion = MathGraphDataInputStream.numeroVersionActif;
		pointeurLongueurUnite = pointeurLongueurUniteInit;
		uniteAngle = uniteAngleInit;
		documentProprietaire = null;
		basseDefinition = true;
	}

	public CListeObjets(CMathGrafDoc documentProprietaireInit) {
		super(20);
		documentProprietaire = documentProprietaireInit;
		numeroVersion = MathGraphDataInputStream.numeroVersionActif;
		basseDefinition = true;
		pointeurLongueurUnite = null;
	}

	// Constructeur avec la mme unit de longueur et la mme longueur unit que
	// list
	public CListeObjets(CListeObjets list) {
		super(20);
		numeroVersion = MathGraphDataInputStream.numeroVersionActif; // Correction bug version 4.1.1.0
		pointeurLongueurUnite = list.pointeurLongueurUnite;
		uniteAngle = list.uniteAngle;
		basseDefinition = true;
	}

	// Met  jour l'unit d'angle et la macro par dfaut aprs cration
	public final void initialise(int uniteAngleInit, CLongueur pointeurLongueurUniteInit) {
		pointeurLongueurUnite = pointeurLongueurUniteInit;
		uniteAngle = uniteAngleInit;
	}

	// Mthode renvoyant true si au moins un des objets de la liste dpend de ptel
	public final boolean dependDe(CElementBase ptel) {
		boolean resultat = false;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext() && !resultat) {
			CElementBase elb = li.next();
			resultat = resultat || elb.dependDe(ptel);
		}
		return resultat;
	}

	public final boolean dependDePourBoucle(CElementBase ptel) {
		boolean resultat = false;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext() && !resultat) {
			CElementBase elb = li.next();
			resultat = resultat || elb.dependDePourBoucle(ptel);
		}
		return resultat;
	}

	// Mthode renvoyant true si et seult si au moins un des lments de la liste
	// existe
	public final boolean existeAuMoinsUnElement() {
		boolean resultat = false;
		ListIterator<CElementBase> li = listIterator();
		// Optimis version 3.4.0.1
		// while (li.hasNext()) {
		while (li.hasNext() && ! resultat) {
			CElementBase elb = li.next();
			resultat = resultat || elb.existe;
		}
		return resultat;
	}
	
	/**
	 * Mthode d'appeler pour chaque objet la mthode metAJour() qui est charge
	 * de mettre  jour tous les lments crs dynamiquement par certians objets
	 * comme les lieux de points et lieux d'objets
	 */

	public final void metAJour() {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase el = li.next();
			el.metAJour();
		}
	}
	

	/**
	 * Ajoute  la liste les lments de la liste list dpendant de l'objet ptel
	 */
	/*
	 * public void AjouteObjetsDependants(CElementBase ptel, CListeObjets list) {
	 * ListIterator<CElementBase> li = list.listIterator(); while (li.hasNext()) {
	 * CElementBase elb = li.next(); if (elb.dependDePourCapture(ptel)) {
	 * add(elb); } } }
	 */

	public final void ajouteObjetsParNature(CListeObjets list, long nature) {
		ListIterator<CElementBase> li = list.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if ((elb.getNature() & nature) != 0)
				add(elb);
		}
	}

	// Ajoute  la liste les lments de la liste list dpendant de l'objet ptel
	// sauf l'lment
	// exclu
	public final void ajouteObjetsDependantsSauf(CElementBase ptel, CListeObjets list, CElementBase exclu) {
		ListIterator<CElementBase> li = list.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if ((elb.dependDePourCapture(ptel)) && (elb != exclu)) {
				add(elb);
			}
		}
	}

  /** Ajoute  la liste  les lments de la liste list dpendant de l'objet ptel
   * Spcifique applet
   * @param ptel
   * @param list
   */
  public void ajouteObjetsDependants(CElementBase ptel, CListeObjets list) {
    ListIterator<CElementBase> li = list.listIterator();
    while (li.hasNext()) {
      CElementBase elb = (CElementBase) (li.next());
      if (elb.dependDePourCapture(ptel)) {
        add(elb);
      }
    }
  }
  
	/**
	 * Mthode d'appeler pour chaque objet dpendant de elb la mthode metAJour()
	 * qui est charge de mettre  jour tous les lments crs dynamiquement par
	 * certians objets comme les lieux de points et lieux d'objets
	 * 
	 * @param elb
	 */
	public final void metAJourObjetsDependantDe(CElementBase elb) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase el = li.next();
			if (el.dependDe(elb))
				el.metAJour();
		}
	}
  
	/**
	 * Ajout version 4.7.2
	 * Mthode d'appeler pour chaque objet dpendant de elb la mthode metAJour() sauf pour l'lment sauf
	 * qui est charge de mettre  jour tous les lments crs dynamiquement par
	 * certains objets comme les lieux de points et lieux d'objets, calcus de drives
	 * Les diteurs de formules eux ne sont pas mis  jour 
	 * 
	 * @param elb
	 */
	public final void metAJourObjetsDependantDeSaufEditeursFormule(CElementBase elb) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase el = li.next();
			if ((!el.estDeNature(NatureObjet.NEditeurFormule)) && el.dependDe(elb))
				el.metAJour();
		}
	}
	
	// Rend tous les objets de la liste visibles ou masques. Les objets doivent
	// tre de type graphique
	public final void montreTout(boolean visible) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if ((elb.getNature() & NatureObjet.NToutObjet) != 0) {
				CElementGraphique elg = (CElementGraphique) elb;
				if (visible)
					elg.montre();
				else
					elg.cache();
			}
		}
	}

	public final void retireTout() {
		int s = size();
		for (int i = 0; i < s; i++) {
			remove(0);
		}
	}

	// Rajoute tous les lments clons de la liste  la liste listeCible
	// de la liste listeSource
	public final void setCopy(CListeObjets listeCible) {
		listeCible.documentProprietaire = documentProprietaire;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			listeCible.add(elb.getClone(this, listeCible));
		}
		listeCible.uniteAngle = uniteAngle;
		if (pointeurLongueurUnite != null) {
			listeCible.pointeurLongueurUnite = (CLongueur) listeCible.get(indexOf(pointeurLongueurUnite));
		}
		// Ce n'est qu'une fois tous les lments clons qu'on peut tablir
		// certaines listes
		// maintenues de faon interne par certaines macros
		listeCible.etablitListesInternesMacros();
	}

	// Cette mthode fait de cette liste dans listeProprietaireCible l'quivalent
	// de listeSource dans la
	// ListePropretaireSource. Bien sur ListeProprietaireCible doit tre une liste
	// clone
	// auparavant de la listeProprietaireSource.
	public final void setImage(CListeObjets listeSource, CListeObjets listeProprietaireSource,
			CListeObjets listeProprietaireCible) {
		ListIterator<CElementBase> li = listeSource.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			int indexSource = listeProprietaireSource.indexOf(elb);
			CElementBase elbCible = (CElementBase) listeProprietaireCible.get(indexSource, CElementBase.class);
			add(elbCible);
		}
		uniteAngle = listeSource.uniteAngle;
		if (listeSource.pointeurLongueurUnite != null) {
			pointeurLongueurUnite = (CLongueur) get(listeSource.indexOf(listeSource.pointeurLongueurUnite));
		}
	}

	/**
	 * Mthode faisant de la liste en cours la liste clone de listeSource
	 * Auparavant, la liste en cours doit avoir t cre comme copie de listeSource  l'aide de setCopy
	 * @param listeSource
	 */
	// Modifi version 4.6
	public final void setClone(CListeObjets listeSource) {
		ListIterator<CElementBase> li = listeSource.listIterator();
		int index = 0;
		while ((index < size()) && li.hasNext()) {
			CElementBase elbSource = li.next();
			CElementBase elb = (CElementBase) get(index);
			elb.setClone(elbSource);
			index++;
		}
	}

	/** Positionnement de tous les objets de la liste */
	public final void positionne(boolean infoRandom, DimensionFenetre dimfen) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			elb.positionne(infoRandom, dimfen);
		}
	}
	
	public final void positionneDependantsDe(boolean infoRandom, DimensionFenetre dimfen, CElementBase pt) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.dependDe(pt))
				elb.positionne(infoRandom, dimfen);
		}
	}	

	/** Affichage de tous les lments affichables de la liste */
	public final void afficheTout(Graphics2D pDC, DimensionFenetre dimfen, boolean masquage, Color couleurFond, float transparence,
			boolean antialiasing) {
    pDC.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasing ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF);
    pDC.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, antialiasing ? RenderingHints.VALUE_TEXT_ANTIALIAS_ON : RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.existe) {
				long nature = elb.getNature();
				if (((nature & NatureObjet.NToutObjet) != 0) || (elb.getNatureCalcul() == NatureCalcul.NRepere)) {
					if (!elb.estElementIntermediaire()) {
						CElementGraphique elg = (CElementGraphique) elb;
						elg.affiche(pDC, masquage, couleurFond, transparence);
						if ((nature & NatureObjet.NObjetNommable) != 0) {
							// Modifi version 3.4.0 pour que quand le rideau est actif, les noms ou droites masqus 
							// soient quand mme affichs.
							if (!masquage || (masquage && !elg.masque)) elg.afficheNom(pDC);
						}
					}
				}
			}
		}
	}
	
	/** Version 4.4 : Affichage de tous les points de la liste marqus pour laisser une trace */
	public final void affichePointsMarquesPourTrace(Graphics2D pDC, DimensionFenetre dimfen, boolean masquage, Color couleurFond,
			float transparence, boolean antialiasing) {
    pDC.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasing ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF);
    pDC.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, antialiasing ? RenderingHints.VALUE_TEXT_ANTIALIAS_ON : RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.existe) {
				if (elb.estDeNature(NatureObjet.NToutPoint) && !elb.estElementIntermediaire()) {
					CPointAncetre pt = (CPointAncetre) elb;
					if (pt.marquePourTrace) pt.affiche(pDC, masquage, couleurFond, transparence);
				}
			}
		}
	}
	

	/** Affichage de tous les lments affichables de la liste sauf ceux de listeExclue */
	public final void afficheToutSaufElementsDe(Graphics2D pDC, DimensionFenetre dimfen, boolean masquage,
			Color couleurFond, float transparence, CListeObjets listeExclue, boolean antialiasing) {
    pDC.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasing ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF);
    pDC.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, antialiasing ? RenderingHints.VALUE_TEXT_ANTIALIAS_ON : RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.existe) {
				long nature = elb.getNature();
				if ((nature & NatureObjet.NToutObjet) != 0) {
					if (!elb.estElementIntermediaire() && !listeExclue.contains(elb)) {
						CElementGraphique elg = (CElementGraphique) elb;
						elg.affiche(pDC, masquage, couleurFond, transparence);
						if ((nature & NatureObjet.NObjetNommable) != 0) {
							// Modifi version 3.4.0 pour que quand le rideau est actif, les noms ou droites masqus 
							// soeint quand mme affichs.
							if (!masquage || (masquage && !elg.masque)) elg.afficheNom(pDC);
						}
					}
				}
			}
		}
	}	
	
	// 
	// les noms)
	/**
	 * Affichage de tous les lments affichables de la liste mme masqus (sans
	 * les noms) Utilis pour les macres avec traces
	 * 
	 */
	public final void afficheToutMemeMasquesPourTrace(Graphics2D pDC, DimensionFenetre dimfen,
			Color couleurFond, float transparence, boolean antialiasing) {
    pDC.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasing ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF);
    pDC.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, antialiasing ? RenderingHints.VALUE_TEXT_ANTIALIAS_ON : RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementGraphique elg = (CElementGraphique) (li.next());
			if (elg.existe) {
				long nature = elg.getNature();
				if (((nature & NatureObjet.NToutObjet) != 0) || (elg.getNatureCalcul() == NatureCalcul.NRepere)) {
					if (!elg.estElementIntermediaire()) {
						boolean estMasque = elg.masque;
						elg.masque = false;
						elg.affiche(pDC, true, couleurFond, transparence);
						elg.masque = estMasque;
					}
				}
			}
		}
	}

	/**
	 * Affichage de tous les lments affichables de la liste mme masqus (avec les noms)
	 * 
	 */
	public final void afficheToutMemeMasques(Graphics2D pDC, DimensionFenetre dimfen, Color couleurFond, float transparence, boolean antialiasing) {
		ListIterator<CElementBase> li = listIterator();
    pDC.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasing ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF);
    pDC.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, antialiasing ? RenderingHints.VALUE_TEXT_ANTIALIAS_ON : RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		while (li.hasNext()) {
			CElementBase elb = (CElementBase) (li.next());
			if ((elb.getNature() & NatureObjet.NToutObjet) != 0) {
				CElementGraphique elg = (CElementGraphique) elb;
				if (elg.existe) {
					long nature = elg.getNature();
					if ((nature & NatureObjet.NToutObjet) != 0) {
						if (!elg.estElementIntermediaire()) {
							boolean estMasque = elg.masque;
							elg.masque = false;
							elg.affiche(pDC, true, couleurFond, transparence);
							elg.masque = estMasque;
							if ((nature & NatureObjet.NObjetNommable) != 0) {
								boolean nomMasque = elg.nomMasque;
								elg.nomMasque = false;
								elg.afficheNom(pDC);
								elg.nomMasque = nomMasque;
							}
						}
					}
				}
			}
		}
	}		

	/**
	 * Fonction affichant tous les lments de la liste jusqu' l'lment d'indice
	 * indDer. Utilis dans l'historique
	 * N'affiche pas les lments masqus
	 * Attention : Fonctionnement diffrent de la version logiciel car pas d'outil gomme
	 * 
	 * @param pDC
	 * @param dimfen
	 * @param indDer
	 */
	public final void afficheJusque(Graphics2D pDC, DimensionFenetre dimfen, int indDer, boolean intermPermis,
			Color couleurFond, float transparence, boolean antialiasing) {
    pDC.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasing ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF);
    pDC.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, antialiasing ? RenderingHints.VALUE_TEXT_ANTIALIAS_ON : RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		int ind = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext() && (ind <= indDer)) {
			CElementBase elb = li.next();
			ind++;
			if (elb.existe) {
				long nature = elb.getNature();
				if (((nature & NatureObjet.NToutObjet) != 0) || (elb.getNatureCalcul() == NatureCalcul.NRepere)) {
					if (!elb.estElementIntermediaire() || intermPermis) {
						CElementGraphique elg = (CElementGraphique) elb;
						elg.affiche(pDC, true, couleurFond, transparence);
						if ((nature & NatureObjet.NObjetNommable) != 0) {
							elg.afficheNom(pDC);
						}
					}
				}
			}
		}
	}

	/**
	 * Fonction affichant tous les lments de la liste jusqu' l'lment d'indice
	 * indDer sauf ceux contenus dans listeEclue. Utilis dans l'historique
	 * 
	 * @param pDC
	 * @param dimfen
	 * @param indDer
	 */
	public final void afficheJusqueSaufElementsDe(Graphics2D pDC, DimensionFenetre dimfen, int indDer,
			CListeObjets listeExclue, boolean intermPermis, Color couleurFond, float transparence, boolean antialiasing) {
    pDC.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasing ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF);
    pDC.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, antialiasing ? RenderingHints.VALUE_TEXT_ANTIALIAS_ON : RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		int ind = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext() && (ind <= indDer)) {
			CElementBase elb = li.next();
			ind++;
			if ((elb.existe) && !listeExclue.contains(elb)) {
				long nature = elb.getNature();
				if ((nature & NatureObjet.NToutObjet) != 0) {
					if (!elb.estElementIntermediaire() || intermPermis) {
						CElementGraphique elg = (CElementGraphique) elb;
						boolean masque = elg.masque;
						elg.masque = false;
						elg.affiche(pDC, true, couleurFond, transparence);
						if ((nature & NatureObjet.NObjetNommable) != 0) {
							boolean nomMasque = elg.nomMasque;
							elg.nomMasque = false;
							elg.afficheNom(pDC);
							elg.nomMasque = nomMasque;
						}
						elg.masque = masque;
					}
				}
			}
		}
	}

	/**
	 * Fonction renvoyant le nombre d'objets de type typeCherche proche du point
	 * de coordonnes point  l'exclusion des objets de la liste
	 * listeExclusionDesignation
	 * 
	 * @param typeCherche
	 * @param point
	 * @param info
	 * @param listeExclusion
	 * @return
	 */
	public final int procheDe(long typeCherche, Point point, InfoProx info, CListeObjets listeExclusion) {
		int ind, indnat;
		long nat;
		CElementBase ptel;
		CElementGraphique ptelg;
		double dis;

		// lorsque la liste est vide, aucun objet n'est proche
		if (size() == 0) {
			return 0;
		}
		info.MetAZero();
		// On traite tous les lments de la liste

		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			ptel = li.next();
			if (listeExclusion.indexOf(ptel) == -1) {
				nat = ptel.getNature();
				if ((nat & typeCherche) != 0) {
					// Modification par rapport au C++
					ptelg = (CElementGraphique) ptel;
					if (!ptelg.masque) {
						dis = ptelg.distancePoint(point.x, point.y, true);
						// LPtoDP(Dc,Poindis,1);
						if ((dis != -1) && (dis < distanceMiniPourEtreProche)) // distance
																																		// infrieure
																																		//  4
																																		// pixels
																																		// pour
						// tre considr comme proche
						{
							// Recherche de l'indice dans le tableau Tableau
							// correspondant  nat
							info.typesProches |= nat;
							indnat = NatureObjet.indiceDeNature(nat);

							info.infoParType[indnat].nombreVoisins++;
							if ((nat & NatureObjet.NToutPoint) != 0) // cas d'un point
							{
								info.nombrePointsVoisins++;
								if (info.premierPointVoisinRencontre) {
									info.premierPointVoisin = (CPointAncetre) ptelg;
									info.premierPointVoisinRencontre = false;
								} else {
									info.dernierPointVoisin = (CPointAncetre) ptelg;
								}
							}

							if (info.infoParType[indnat].premierVoisinRencontre) {
								info.infoParType[indnat].premierVoisin = ptelg;
								info.infoParType[indnat].premierVoisinRencontre = false;
							} else {
								info.infoParType[indnat].dernierVoisin = ptelg;
							}
						} // if Poindis
					} // if (!Ptel.Masque)
				} // if (nat and type_cherche <>0)
			} // if listeExclusion
		}// Fin de la boucle de traitement de chacun des lments de la liste

		// Priorits aux objets points sauf si on est proche d'un lieu
		// de points
		if (info.nombrePointsVoisins > 0) {
			info.typesProches &= NatureObjet.NToutPoint;
		}
		info.nombreObjetsProches = 0;
		for (ind = 0; ind < NatureObjet.nombreTypesDesignables; ind++) {
			info.nombreObjetsProches += info.infoParType[ind].nombreVoisins;
		}
		info.nombreTypesProches = 0;
		for (ind = 0; ind < NatureObjet.nombreTypesDesignables; ind++) {
			if ((info.typesProches & NatureObjet.conversionNumeroNature[ind]) != 0) {
				info.nombreTypesProches++;
			}
		}
		return info.nombreObjetsProches;
	}

	/**
	 * Fonction renvoyant le nombre d'objets de type typeCherche proches du point
	 * de coordonnes point et possdant un dialogue permettant de les modifier 
	 * l'exclusion des objets de la liste listeExclusionDesignation
	 * 
	 * @param typeCherche
	 * @param point
	 * @param info
	 * @param listeExclusion
	 * @return
	 */
	public final int procheDePourDialogueModification(long typeCherche, Point point, InfoProx info,
			CListeObjets listeExclusion) {
		int ind, indnat;
		long nat;
		CElementBase ptel;
		CElementGraphique ptelg;
		double dis;

		// lorsque la liste est vide, aucun objet n'est proche
		if (size() == 0) {
			return 0;
		}
		info.MetAZero();
		// On traite tous les lments de la liste

		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			ptel = li.next();
			if (ptel.modifiableParMenu() && (listeExclusion.indexOf(ptel) == -1)) {
				nat = ptel.getNature();
				if ((nat & typeCherche) != 0) {
					// Modification par rapport au C++
					ptelg = (CElementGraphique) ptel;
					if (!ptelg.masque) {
						dis = ptelg.distancePoint(point.x, point.y, true);
						// LPtoDP(Dc,Poindis,1);
						if ((dis != -1) && (dis < distanceMiniPourEtreProche)) // distance
																																		// infrieur
																																		//  4
																																		// pixels
																																		// pour
						// tre considr comme proche
						{
							// Recherche de l'indice dans le tableau Tableau
							// correspondant  nat
							info.typesProches |= nat;
							indnat = NatureObjet.indiceDeNature(nat);

							info.infoParType[indnat].nombreVoisins++;
							if ((nat & NatureObjet.NToutPoint) != 0) // cas d'un point
							{
								info.nombrePointsVoisins++;
								if (info.premierPointVoisinRencontre) {
									info.premierPointVoisin = (CPointAncetre) ptelg;
									info.premierPointVoisinRencontre = false;
								} else {
									info.dernierPointVoisin = (CPointAncetre) ptelg;
								}
							}

							if (info.infoParType[indnat].premierVoisinRencontre) {
								info.infoParType[indnat].premierVoisin = ptelg;
								info.infoParType[indnat].premierVoisinRencontre = false;
							} else {
								info.infoParType[indnat].dernierVoisin = ptelg;
							}
						} // if Poindis
					} // if (!Ptel.Masque)
				} // if (nat and type_cherche <>0)
			} // if listeExclusion
		}// Fin de la boucle de traitement de chacun des lments de la liste

		// Priorits aux objets points sauf si on est proche d'un lieu
		// de points
		if (info.nombrePointsVoisins > 0) {
			info.MetAZeroObjetsNonPoints(); // Ajout version 2.7
			info.typesProches &= NatureObjet.NToutPoint; // Car un repre se dsigne par son origine ds la version java
		}
		info.nombreObjetsProches = 0;
		for (ind = 0; ind < NatureObjet.nombreTypesDesignables; ind++) {
			info.nombreObjetsProches += info.infoParType[ind].nombreVoisins;
		}
		info.nombreTypesProches = 0;
		for (ind = 0; ind < NatureObjet.nombreTypesDesignables; ind++) {
			if ((info.typesProches & NatureObjet.conversionNumeroNature[ind]) != 0) {
				info.nombreTypesProches++;
			}
		}
		return info.nombreObjetsProches;
	}	
	// Mthode tablissant certaines listes maintenues de faon interne par
	// certaines macros
	public final void etablitListesInternesMacros() {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			long nature = elb.getNature();
			if (nature == NatureObjet.NMacro) {
				((CMacro) elb).etablitListesInternes();
			}
		}
	}

	// Renvoie null si Intitule n'est pas l'intitul d'une macro ou sinon
	// renvoie un pointeur sur cette macro
	public final CMacro pointeurMacro(String intitule) {
		CMacro resultat = null;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext() && (resultat == null)) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NMacro) {
				CMacro mac = (CMacro) elb;
				if (mac.intitule.equals(intitule)) {
					resultat = mac;
				}
			}
		}
		return resultat;
	}
	
	// Ajoute  la liste les images
	public final void ajouteImagesOldVersion(CListeObjets list) {
		ListIterator<CElementBase> li = list.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NImage) {
				if (((CImage) elb).isOldVersion) add(elb);
			}
		}
	}	

	// Ajoute  la liste les macros jouant un fichier sonore contenues dans list
	public final void ajouteMacrosJouantSon(CListeObjets list) {
		ListIterator<CElementBase> li = list.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NMacro) {
				if (elb.getClass() == CMacroJouantSon.class) {
					add(elb);
				}
			}
		}
	}

	/**
	 * Utilis dans CCommentaire pour savoir si la chaine utilise reprsente une
	 * valeur relle dfinie avant un certain indice Si indiceMaxi vaut -1, on
	 * recherche dans tous les objets Utilis aussi dans les diteurs de valeurs
	 * complexes
	 */
	public final CValeurDynamique pointeurValeurComplexe(String nomValeur, int indiceMaxi, Bool definiApres) {
		if (indiceMaxi == -1) {
			indiceMaxi = size() - 1;
		}
		definiApres.setValue(false);
		int i = 0;
		CElementBase elb = null;
		boolean trouve = false;
		ListIterator<CElementBase> li = listIterator();
		while ((!(trouve)) && (i <= indiceMaxi)) {
			elb = li.next();
			if ((elb.getNatureCalcul() & NatureCalcul.NTouteValeurComplexe) != 0) {
				if (((CValeurDynamique) elb).chaineEgaleANom(nomValeur)) {
					trouve = true;
				} else
					i++;
			} else
				i++;
		}
		if (trouve) {
			return (CValeurDynamique) elb;
		} else {
			// On regarde si nomValeur contient le nom d'une valeur d'indice suprieur
			//  indiceMaxi
			i = indiceMaxi + 1;
			while ((!(trouve)) && (i < size())) {
				elb = li.next();
				if ((elb.getNatureCalcul() & NatureCalcul.NTouteValeurComplexe) != 0) {
					if (((CValeurDynamique) elb).chaineEgaleANom(nomValeur)) {
						trouve = true;
					} else
						i++;
				} else
					i++;
			}
			if (trouve)
				definiApres.setValue(true);
			return null;
		}
	}

	// Utilis dans MtgConverter
	/* Supprim version 4.7.2
	public final CValeurDynamique pointeurValeurOuFonction(String nom, int indiceMaxi) {
		if (indiceMaxi == -1) {
			indiceMaxi = size() - 1;
		}
		int i = 0;
		CElementBase elb = null;
		boolean trouve = false;
		ListIterator<CElementBase> li = listIterator();
		while ((!(trouve)) && (i <= indiceMaxi)) {
			elb = li.next();
			if (elb.estDeNatureCalcul(NatureCalcul.NTouteValeurPourCommentaireDynamique)) {
				if (((CValeurDynamique) elb).nom().equals(nom)) {
					trouve = true;
				} else {
					i++;
				}
			} else {
				i++;
			}
		}
		if (trouve) {
			return (CValeurDynamique) elb;
		} else {
			return null;
		}
	}
	*/
	
	/**
	 * Ajout version 4.7.2 pour utilisation dans MtGConverter
	 * Renvoie un pointeur sur le calcul ou la fonction de nom dont le test nat & natureCalcul est non nul
	 * @param nat : le type de calcul cherch (peut contenir plusieurs types avec Or
	 * @param nom : le nom du calcul cherch
	 * @param indiceMaxi : L'indice maxi de recherche dans la liste
	 * @return
	 */
	public CValeurDynamique pointeurParNatureCalcul(long nat,String nom, int indiceMaxi) {
		if (indiceMaxi == -1) {
			indiceMaxi = size() - 1;
		}
		int i = 0;
		CElementBase elb = null;
		boolean trouve = false;
		ListIterator<CElementBase> li = listIterator();
		while ((!(trouve)) && (i <= indiceMaxi)) {
			elb = li.next();
			if (elb.estDeNatureCalcul(nat)) {
				if (((CValeurDynamique) elb).nom().equals(nom)) {
					trouve = true;
				} else {
					i++;
				}
			} else {
				i++;
			}
		}
		if (trouve) {
			return (CValeurDynamique) elb;
		} else return null;
	}
		
	// Utilis dans AppletManager
	// Modifi version 4.6
	public final CCalcul pointeurCalculOuFonction(String nom, int indiceMaxi) {
		if (indiceMaxi == -1) {
			indiceMaxi = size() - 1;
		}
		for (int i = 0; i <= indiceMaxi; i++) {
			CElementBase elb = get(i);
			if (elb.estDeNatureCalcul(NatureCalcul.NCalculReel | NatureCalcul.NFonction | 
					NatureCalcul.NFonction2Var | NatureCalcul.NFonction3Var)) {
				if (((CValeurDynamique) elb).nom().equals(nom)) return (CCalcul) elb;
			}
		}
		return null;
	}
	
	// Utilis dans DonneValeurA de l'applet
	// Modifi version 4.6
	public final CCalcul pointeurCalculOuFonctionComplexe(String nom, int indiceMaxi) {
		if (indiceMaxi == -1) {
			indiceMaxi = size() - 1;
		}
		for (int i = 0; i <= indiceMaxi; i++) {
			CElementBase elb = get(i);
			if (elb.estDeNatureCalcul(NatureCalcul.NCalculComplexe | NatureCalcul.NFonctionComplexe
					| NatureCalcul.NFonctionComplexe2Var | NatureCalcul.NFonctionComplexe3Var)) {
				if (((CValeurDynamique) elb).nom().equals(nom)) {
					return (CCalcul)elb;
				}
			}
		}
		return null;
	}	

	// Ajout version 4.6
	/**
	 * Utilise dans valeurDe . Renvoie un pointeur sur le calcul nomm rel de nom nom
	 * Si pas trouv renvoie null
	 * @param nom
	 * @param indiceMaxi
	 * @return
	 */
	public final CCalculAncetre pointeurCalculReelNomme(String nom, int indiceMaxi) {
		if (indiceMaxi == -1) {
			indiceMaxi = size() - 1;
		}
		CElementBase elb = null;
		for (int i = 0; i <= indiceMaxi; i++) {
			elb = get(i);
			if (elb.estDeNatureCalcul(NatureCalcul.NToutCalculReelNommeSaufConst)) {
				if (((CCalculAncetre) elb).nomCalcul.equals(nom)) {
					return (CCalculAncetre) elb;
				}
			}
		}
		return null;
	}

	/**
	 * Fonction renvoyant 0 si la chaine chaine pointe par PChaine ne commence
	 * pas par un nom de valeur relle et sinon renvoie la longueur du nom de
	 * cette valeur et dans PointeurValeur un pointeur sur celle-ci. Le dernier
	 * paramtre indique si on admet les lments intermdiaires de construction
	 */
	public final int commenceParNomValeur(String pChaine, PointeurValDyn pointeurValeur) {
		CValeurDynamique ptv;
		Point longueurNom = new Point();

		int longueurTrouvee = 0;

		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (!elb.estElementIntermediaire() && ((elb.getNatureCalcul() & NatureCalcul.NTouteValeurReelle) != 0)) {
				ptv = (CValeurDynamique) elb;
				if (ptv.chaineCommenceParNom(pChaine, longueurNom) && (longueurNom.x > longueurTrouvee)) {
					if (!ptv.estElementIntermediaire()) {
						longueurTrouvee = longueurNom.x;
						pointeurValeur.setPointeur(ptv);
					}
				}
			}
		} // while
		return longueurTrouvee;
	}

	/**
	 * Fonction renvoyant 0 si la chaine chaine pointe par PChaine ne commence
	 * pas par un nom de valeur relle et sinon renvoie la longueur du nom de
	 * cette valeur et dans PointeurValeur un pointeur sur celle-ci Le dernier
	 * paramtre indique si on admet les lments intermdiaires de construction
	 */
	public final int commenceParNomValeurReelleOuComplexe(String pChaine, PointeurValDyn pointeurValeur) {
		CValeurDynamique ptv;
		Point longueurNom = new Point();

		int longueurTrouvee = 0;

		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (! elb.estElementIntermediaire() && ((elb.getNatureCalcul() & NatureCalcul.NTouteValeurReelleOuComplexe) != 0)) {
				ptv = (CValeurDynamique) elb;
				if (ptv.chaineCommenceParNom(pChaine, longueurNom) && (longueurNom.x > longueurTrouvee)) {
					if (!ptv.estElementIntermediaire()) {
						longueurTrouvee = longueurNom.x;
						pointeurValeur.setPointeur(ptv);
					}
				}
			}
		} // while
		return longueurTrouvee;
	}

	/**
	 * Fonction renvoyant 0 si la chaine pointe par PChaine ne commence pas par
	 * un nom de fonction ou de suite rcurrente et sinon renvoie la longueur du
	 * nom de cette fonction ou suite et dans PointeurFonction un pointeur sur
	 * celle-ci
	 */
	public final int commenceParNomFonctionUtilisateur(String pChaine, PointeurValDyn pointeurFonction) {
		CCalculAncetre ptf;
		Point longueurNom = new Point();

		int longueurTrouvee = 0;
		pointeurFonction.setPointeur(null);
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			// Modifi version 4.7.5
			// if (!elb.estElementIntermediaire() && elb.estDeNatureCalcul(NatureCalcul.NTouteFonctionReelle | NatureCalcul.NDerivee | NatureCalcul.NSuiteRecurrenteReelle)) {
			if (!elb.estElementIntermediaire() && elb.estDeNatureCalcul(NatureCalcul.NTouteFonctionReelle
					| NatureCalcul.NDerivee | NatureCalcul.NTouteSuiteRecurrenteReelle)) {
				ptf = (CCalculAncetre) elb;
				if (ptf.chaineCommenceParNom(pChaine, longueurNom) && (longueurNom.x > longueurTrouvee)) {
					longueurTrouvee = longueurNom.x;
					pointeurFonction.setPointeur(ptf);
				}
			}
		}
		return longueurTrouvee;
	}

	/**
	 * Fonction renvoyant 0 si la chaine pointe par PChaine ne commence pas par
	 * un nom de fonction ou de suite rcurrente et sinon renvoie la longueur du
	 * nom de cette fonction ou suite et dans PointeurFonction un pointeur sur
	 * celle-ci
	 */
	public final int commenceParNomFonctionComplexeUtilisateur(String pChaine, PointeurValDyn pointeurFonction) {
		CCalculAncetre ptf;
		Point longueurNom = new Point();

		int longueurTrouvee = 0;
		pointeurFonction.setPointeur(null);
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			// Modifi version 4.7.5
			// if (!elb.estElementIntermediaire() && elb.estDeNatureCalcul(NatureCalcul.NTouteFonctionComplexe | NatureCalcul.NSuiteRecurrenteComplexe)) {
			if (!elb.estElementIntermediaire() && elb.estDeNatureCalcul(NatureCalcul.NTouteFonctionComplexe
					| NatureCalcul.NTouteSuiteRecurrenteComplexe)) {
				ptf = (CCalculAncetre) elb;
				if (ptf.chaineCommenceParNom(pChaine, longueurNom) && (longueurNom.x > longueurTrouvee)) {
					longueurTrouvee = longueurNom.x;
					pointeurFonction.setPointeur(ptf);
				}
			}
		}
		// for
		return longueurTrouvee;
	}

	/**
	 * Fonction renvoyant 0 si la chaine pointe par pChaine ne pointe pas sur une
	 * chaine reprsentant une valeur ou une fonction mathmatique prdfinie ou
	 * une fonction dfinie par l'utilisateur ou le parametre. Sinon, elle renvoie
	 * la longueur du nom le plus long trouve. Si ParametreTrouve est non nul,
	 * c'est qu'on a trouv le paramtre formel et parametreTrouve contient son indice
	 * dans le tableau parametreFormel , sinon, si PointeurValeur est
	 * diffrent de NULL, c'est qu'un a trouv un nom de valeur ou de fonction
	 * dfinie par l'utilisateur (les suites tant considres comme des
	 * fonctions) sinon c'est qu'on a trouv un nom de fonction prdfinie dont
	 * la nature est renvoye dans NomFonction.
	 * Les calculs intermdiaires ne sont pas autoriss
	 */
	public final int commenceParNomFonctionOuValeurOuParametre(String pChaine, String[] parametreFormel,
			PointeurValDyn pointeurValeur, Int nomFonction, Int parametreTrouve, Int nombreVariables) {
		int longueurNomFonctionPredefinie;
		int longueurNomValeur;
		PointeurValDyn pointeurSurValeur = new PointeurValDyn();
		PointeurValDyn pointeurSurFonction = new PointeurValDyn();
		int longueurNomFonctionUtilisateur;
		int indiceParametre = -1;
		int longueurNomParametre = 0;

		parametreTrouve.setValue(-1);
		if (parametreFormel != null) {

			for (int i = 0; i < parametreFormel.length; i++) {
				if (pChaine.startsWith(parametreFormel[i]) && (parametreFormel[i].length() > longueurNomParametre)) {
					longueurNomParametre = parametreFormel[i].length();
					indiceParametre = i;
					parametreTrouve.setValue(i); // Ligne rajoute version 4.5.2
				}
			}
		}
		longueurNomFonctionPredefinie = CCalculBase.commenceParNomFonctionReellePredefinie(pChaine, nomFonction, nombreVariables);
		longueurNomValeur = commenceParNomValeur(pChaine, pointeurSurValeur);
		longueurNomFonctionUtilisateur = commenceParNomFonctionUtilisateur(pChaine, pointeurSurFonction);
		if ((longueurNomFonctionPredefinie == 0) && (longueurNomValeur == 0)
				&& (longueurNomFonctionUtilisateur == 0) && (longueurNomParametre == 0)) {
			return 0;
		} else // Si la chaine commence par un des trois types de noms, un des
		// trois noms est ncessairement strictement suprieur aux deux
		// autres, car il est interdit pour la syntaxe que deux types de
		// noms diffrents de mme longueur aient les mmes caractres
	  // Modification version 3.4.3 : un paramtre formel d'une fonction peut avoir le mme nom
		// qu'un objet calcul dj dfini. Il a alors priorit :  l'utilisateur de ne pas choisir
		// un paramtre formel ayant un nom d utilis
		{
			if ((longueurNomParametre >= longueurNomFonctionPredefinie)
					&& (longueurNomParametre >= longueurNomValeur)
					&& (longueurNomParametre >= longueurNomFonctionUtilisateur)) {
				parametreTrouve.setValue(indiceParametre);
				return longueurNomParametre;
			} else {
				parametreTrouve.setValue(-1); // Correction bug version 46
				if ((longueurNomFonctionPredefinie > longueurNomValeur)
						&& (longueurNomFonctionPredefinie > longueurNomFonctionUtilisateur)
						&& (longueurNomFonctionPredefinie > longueurNomParametre)) {
					pointeurValeur.setPointeur(null);
					return longueurNomFonctionPredefinie;
				} else {
					if ((longueurNomValeur > longueurNomFonctionPredefinie)
							&& (longueurNomValeur > longueurNomFonctionUtilisateur)
							&& (longueurNomValeur > longueurNomParametre)) {
						pointeurValeur.setPointeur(pointeurSurValeur.getPointeur());
						return longueurNomValeur;
					} else {
						if ((longueurNomFonctionUtilisateur > longueurNomFonctionPredefinie)
								&& (longueurNomFonctionUtilisateur > longueurNomValeur)
								&& (longueurNomFonctionUtilisateur > longueurNomParametre)) {
							pointeurValeur.setPointeur(pointeurSurFonction.getPointeur());
							nombreVariables.setValue(((CCalculAncetre)pointeurSurFonction.getPointeur()).nombreVariables());
							return longueurNomFonctionUtilisateur;
						} else {
							return 0;
						} // par scurit
					}
				}
			}
		}
	}

	/**
	 * Fonction renvoyant 0 si la chaine pointe par pChaine ne pointe pas sur une
	 * chaine reprsentant une valeur ou une fonction mathmatique prdfinie ou
	 * une fonction dfinie par l'utilisateur ou le complexe i ou le parametre.
	 * Sinon, elle renvoie la longueur du nom le plus long trouve. Si
	 * ParametreTrouve est  true, c'est qu'on a trouv le paramtre formel,
	 * sinon, si PointeurValeur est diffrent de NULL, c'est qu'un a trouv un nom
	 * de valeur ou de fonction dfinie par l'utilisateur (les suites tant
	 * considres comme des fonctions) sinon c'est qu'on a trouv un nom de
	 * fonction prdfinie dont la nature est renvoye dans NomFonction.
	 * Les calculs intermdiaires ne sont pas autoriss
	 */
	public final int commenceParNomFonctionOuValeurOuParametreComplexe(String pChaine, String parametreFormel[],
			PointeurValDyn pointeurValeur, Int nomFonction, Int parametreTrouve, Bool complexeiTrouve, Int nombreVariables) {
		int longueurNomFonctionPredefinie;
		int longueurNomValeur;
		PointeurValDyn pointeurSurValeur = new PointeurValDyn();
		PointeurValDyn pointeurSurFonction = new PointeurValDyn();
		int longueurNomFonctionUtilisateur;
		int indiceParametre = -1;
		int longueurNomParametre = 0;

		parametreTrouve.setValue(-1);
		complexeiTrouve.setValue(false);
		pointeurValeur.setPointeur(null);

		if (parametreFormel != null) {

			for (int i = 0; i < parametreFormel.length; i++) {
				if (pChaine.startsWith(parametreFormel[i]) && (parametreFormel[i].length() > longueurNomParametre)) {
					longueurNomParametre = parametreFormel[i].length();
					indiceParametre = i;
					parametreTrouve.setValue(i); // Ligne rajoute version 4.5.2
				}
			}
		}
		longueurNomFonctionPredefinie = CCalculBase.commenceParNomFonctionComplexePredefinie(pChaine, nomFonction, nombreVariables);
		longueurNomValeur = commenceParNomValeurReelleOuComplexe(pChaine, pointeurSurValeur);
		longueurNomFonctionUtilisateur = commenceParNomFonctionComplexeUtilisateur(pChaine, pointeurSurFonction);
		if ((longueurNomFonctionPredefinie == 0) && (longueurNomValeur == 0)
				&& (longueurNomFonctionUtilisateur == 0) && (longueurNomParametre == 0)) {
			if (pChaine.startsWith("i")) {
				complexeiTrouve.setValue(true);
				return 1;
			} else
				return 0;

		} else // Si la chaine commence par un des trois types de noms, un des
		// trois noms est ncessairement strictement suprieur aux deux
		// autres, car il est interdit pour la syntaxe que deux types de
		// noms diffrents de mme longueur aient les mmes caractres
		// Modification version 3.4.3 : un paramtre formel d'une fonction peut avoir le mme nom
		// qu'un objet calcul dj dfini. Il a alors priorit :  l'utilisateur de ne pas choisir
		// un paramtre formel ayant un nom d utilis
		{
			if ((longueurNomParametre >= longueurNomFonctionPredefinie)
					&& (longueurNomParametre >= longueurNomValeur)
					&& (longueurNomParametre >= longueurNomFonctionUtilisateur)) {
				parametreTrouve.setValue(indiceParametre);
				return longueurNomParametre;
			} else {
				parametreTrouve.setValue(-1); // Correction bug version 46
				if ((longueurNomFonctionPredefinie > longueurNomValeur)
						&& (longueurNomFonctionPredefinie > longueurNomFonctionUtilisateur)
						&& (longueurNomFonctionPredefinie > longueurNomParametre)) {
					return longueurNomFonctionPredefinie;
				} else {
					if ((longueurNomValeur > longueurNomFonctionPredefinie)
							&& (longueurNomValeur > longueurNomFonctionUtilisateur)
							&& (longueurNomValeur > longueurNomParametre)) {
						// On teste si on fait rfrence  un calcul ou une variable nomm i.
						// Si c'est le cas, on donne priorit au complexe i
						if ((longueurNomValeur == 1) && pChaine.startsWith("i")) {
							complexeiTrouve.setValue(true);
							return 1;
						} else {
							pointeurValeur.setPointeur(pointeurSurValeur.getPointeur());
							return longueurNomValeur;
						}
					} else {
						if ((longueurNomFonctionUtilisateur > longueurNomFonctionPredefinie)
								&& (longueurNomFonctionUtilisateur > longueurNomValeur)
								&& (longueurNomFonctionUtilisateur > longueurNomParametre)) {
							nombreVariables.setValue(((CCalculAncetre)pointeurSurFonction.getPointeur()).nombreVariables());
							pointeurValeur.setPointeur(pointeurSurFonction.getPointeur());
							return longueurNomFonctionUtilisateur;
						} else {
							return 0;
						} // par scurit
					}
				}
			}
		}
	}

	/*
	 * Fonction renvoyant true si un objet confondu avec l'objet point par ptel a
	 * dj t cr et false sinon. Dans la version Applet, confonduAvec() n'est
	 * pas redfini pour tous les objets mais seulement pour ceux qui peuvent tre
	 * crs  l'intrieur de l'applet
	 */

	public final boolean existeDeja(CElementBase ptel) {
		ListIterator<CElementBase> li = listIterator();
		boolean res = false;
		while (li.hasNext() && (!res)) {
			CElementBase elb = li.next();
			res = res || elb.confonduAvec(ptel);
		}
		return res;
	}
	
	/**
	 * Fonction renvoyant null si la liste ne contient pas de point image de pt
	 * par la transformation transf et sinon un pointeur sur ce point image
	 * 
	 * @param pt
	 * @param transf
	 * @return
	 */
	public final CPointImage pointImageDePar(CPointAncetre pt, CTransformation transf) {
		CPointImage im = null;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NPoint) {
				if (((CPointAncetre)elb).estPointImage()) {
					CPointImage ptim = (CPointImage) elb;
					if ((ptim.transformation == transf) && (ptim.antecedent == pt)) {
						im = ptim;
						break;
					}
				}
			}
		}
		return im;
	}	

	/*
	 * Fonction dtruisant le dernier objet de la liste
	 */

	public final void detruitDernierObjet() {
		int ind = size() - 1;
		if (ind >= 0) {
			remove(ind);
		}
	}

	/*
	 * Fonction remplaant dans tous les objets de la liste l'objet ancienPoint
	 * par l'objet nouveauPoint
	 */

	public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			elb.remplacePoint(ancienPoint, nouveauPoint);
		}
	}

	/**
	 * Mthode renvoyant le nombre d'objets dpendant de l'objet pt
	 */
	public final int nombreDependants(CElementBase pt) {
		int res = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.dependDe(pt)) {
				res++;
			}
		}
		return res;
	}

	/**
	 * Mthode retirant le la liste les objets dpendant de pt Pour cela la liste
	 * est parcourue de la fin vers le dbut
	 */
	public final void retireDependants(CElementBase pt) {
		boolean existeImplementationsProtoOrphelines = false;
		int index = size() - 1;
		while (index >= 0) {
			CElementBase elb = get(index);
			if (elb.dependDe(pt)) {
				if (elb == pointeurLongueurUnite) pointeurLongueurUnite = null;
				if (elb.estElementFinal) {
					elb.impProto.nbFinaux--;
					// Ajout version 3.6.1 pour corriger un bug. En effet le nombre d'objets intermdiaires
					// doitt aussi tre actualis car le reclassement d'objets l'utilise
					elb.impProto.determineNombreObjetsIntermediaires();
					existeImplementationsProtoOrphelines = existeImplementationsProtoOrphelines
							|| (elb.impProto.nbFinaux == 0);
				}
				remove(elb);
			}
			// Modification version 3.3.3.1 ligne suivante pour optimiser
			if (elb == pt) break;
			index--;
		}
		if (existeImplementationsProtoOrphelines)
			retireImplementationsProtoOrphelines();
	}

	public final void retireNDerniersElements(int N) {
		for (int i = 0; i < N; i++) {
			detruitDernierObjet();
		}
	}

	/**
	 * Quand on a dtruit le dernier lment final d'une implmentation de
	 * Prototype Elle devient inutile et doit tre supprime
	 */
	private final void retireImplementationsProtoOrphelines() {
		for (int i = 0; i < size(); i++) {
			CElementBase elb = get(i);
			if (elb.getNature() == NatureObjet.NImplementationProto) {
				CImplementationProto imp = (CImplementationProto) elb;
				if (imp.nbFinaux == 0) {
					retireDependants(imp);
					i = 0; // On recommence depuis le dbut
				}
			}
		}
	}

	/**
	 * Fonction rajoute pour la version 1.9.5 Cre une liste d'objets forme de
	 * NbCopies de l'objet ptelg Ces objets ne servent qu' mmoriser la position
	 * d'autres objets lors du positionnement des lieux d'objets ListeSource
	 */
	public final void ajouteClonesDe(CElementGraphique ptelg, int nombreCopies) {
		for (int i = 0; i < nombreCopies; i++) {
			add(ptelg.creeClone());
		}
	}

	// Ajout pour la version 2.0
	/*
	 * redfinition de gte de faon que si l'argument est gal  -1 la valeur
	 * renvoye soir null
	 */
	public CElementBase get(int indice) {
		if (indice == -1) {
			return null;
		} else {
			return (CElementBase) super.get(indice);
		}
	}

	/*
	 * Ajout pour la version 2.5 pour le chargement des prototypes. cla contient
	 * la classe attendue pour l'objet renvoy& Si l'lment renvoy par get() est
	 * du type CElementGenerique, on va remplacer dans list cet lmnt par un
	 * nouvel lment de type cla sinon on renvoie un pointeur du type souhait
	 * vers l'lment d'indice indice de list
	 */

	@SuppressWarnings({ "rawtypes" })
	public CElementBase get(int indice, Class cla) {
		CElementBase nouv = null;
		if (indice == -1) {
			return null;
		}
		CElementBase elb = (CElementBase) super.get(indice);
		if (elb.getClass() == CElementGenerique.class) {
			// On remplace le CElementGenerique par un nouvel objet de classe cla
			try {
				nouv = (CElementBase) cla.newInstance();
				nouv.elementGeneriqueAssocie = (CElementGenerique) elb;
			} catch (InstantiationException e) {
			} catch (IllegalAccessException e) {
			}
			// On leurre le prototype en lui renvoyant un pointeur sur un lment pas
			// rellement implant;
			return nouv;
		} else {
			return (CElementBase) super.get(indice);
		}
	}

	/**
	 * // Si la formule d'une fonction a t modifie via DonneValeurA dans //
	 * AppletMathGraph32 // il faut vrifier si une drive de cette fonction a
	 * t cre et si oui // recalculer cette drive et si une drivee de cette
	 * drive // a t cre il faut recommencer et ainsi de suite. // La
	 * procdure doit donc tre rcursive / indiceDebut est le premier indice 
	 * partir duquel commence la recherche
	 * Modifi version 2.0 : fon pointera soit sur une CFonc soit sur une CFoncNVar
	 */
	public final void recalculeDeriveesDependantDe(CCalcul fon) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNatureCalcul() == NatureCalcul.NDerivee) {
				CFonc ptfon = (CFonc) elb;
				if (ptfon.getClass() == CDerivee.class) {
					CDerivee ptder = (CDerivee) ptfon;
					if (ptder.dependDe(fon)) {
						// Au chargement d'une drive, depuis l'introduction de CProduitDansFormule et CSommeDansFormule
						// il faut vrifier que le calcul est possible. Si ce n'est pas le cas, on cre une division
						// par 0 pour que la drive n'existe pas.
						if (! ptder.fonctionAssociee.deriveePossible(0)) 
							ptder.calcul = new COperation(ptder.listeProprietaire, new CConstante(ptder.listeProprietaire, 1),
									new CConstante(ptder.listeProprietaire, 0), Operateur.Divi);
						else ptder.creeCalcul();

					}
				}
			}
		}
	}	
	// Chargement de la liste depuis un flux
	public void chargeDepuisFlux(MathGraphDataInputStream inps) throws IOException {
		// On lit d'abord l'unit d'angle
		uniteAngle = inps.readCShort();
		// On lit d'abord le nombre d'objets de la liste
		int nombreObjets = inps.readCCount();
		for (int i = 0; i < nombreObjets; i++) {
			CElementBase elb = (CElementBase) inps.readCObject(this);
			add(elb);
		}
		// Il est maintenant inutile se sauvegarder dans le flux l'indice de la
		// longueur unit car c'est lors
		// du chargement de la premire longueur que celle-ci informa la liste
		// qu'elle est la longueur unit
		@SuppressWarnings("unused")
		int indiceLongueurUnite = inps.readCInt();
		/*
		 * if (indiceLongueurUnite == -1) { pointeurLongueurUnite = null; } else {
		 * pointeurLongueurUnite = (CLongueur) get(indiceLongueurUnite); }
		 */
		int indiceMacroParDefaut = inps.readCInt();
		if (indiceMacroParDefaut != -1) {
			macroParDefaut = (CMacro) get(indiceMacroParDefaut);
		}
		//  revoir en java une fois les macros dfinies
		// Certaines macros qui maintiennent une liste interne d'objets doivent
		// tablir cette liste
		//  ce niveau
		etablitListesInternesMacros();
	}

	// Ajout pour la version 2.5 pour le chargement des prototypes
	public void chargePourProtoDepuisFlux(MathGraphDataInputStream inps) throws IOException {
		int nombreObjets = inps.readCCount();
		for (int i = 0; i < nombreObjets; i++) {
			CElementBase elb = (CElementBase) inps.readCObject(this);
			add(elb);
		}
	}

	/* Chargement de la liste depuis un flux java */
	public void read(MathGraphDataInputStream inps) throws IOException {
		// On lit d'abord l'unit d'angle
		uniteAngle = inps.readInt();
		// On lit d'abord le nombre d'objets de la liste
		int nombreObjets = inps.readInt();
		for (int i = 0; i < nombreObjets; i++) {
			CElementBase elb = (CElementBase) inps.readObject(this);
			add(elb);
		}
		int indiceLongueurUnite = inps.readInt();
		if (indiceLongueurUnite == -1) {
			pointeurLongueurUnite = null;
		} else {
			pointeurLongueurUnite = (CLongueur) get(indiceLongueurUnite);
		}
		// Inutile dans l'applet
		int indiceMacroParDefaut = inps.readInt();
		if (indiceMacroParDefaut != -1) {
			macroParDefaut = (CMacro) get(indiceMacroParDefaut);
		}
		//  revoir en java une fois les macros dfinies
		// Certaines macros qui maintiennent une liste interne d'objets doivent
		// tablir cette liste
		//  ce niveau
		etablitListesInternesMacros();
	}

	
	/* Enregistrement de la liste dans un flux java */
	public void write(MathGraphDataOutputStream oups) throws IOException {
		oups.writeInt(uniteAngle);
		oups.writeInt(size());
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			oups.writeObject(elb);
		}
		int indiceLongueurUnite;
		if (pointeurLongueurUnite == null)
			indiceLongueurUnite = -1;
		else
			indiceLongueurUnite = indexOf(pointeurLongueurUnite);
		oups.writeInt(indiceLongueurUnite);
		// Inutile pour l'applet
		int indiceMacroParDefaut;
		if (macroParDefaut == null)
			indiceMacroParDefaut = -1;
		else
			indiceMacroParDefaut = indexOf(macroParDefaut);
		oups.writeInt(indiceMacroParDefaut);

	}

	// Pour le chargement des prototypes
	public void readPourProtoDepuisFlux(MathGraphDataInputStream inps) throws IOException {
		int nombreObjets = inps.readInt();
		for (int i = 0; i < nombreObjets; i++) {
			CElementBase elb = (CElementBase) inps.readObject(this);
			add(elb);
		}
	}

	// Pour l'enregistrement des prototypes
	public void writePourProtoDepuisFlux(MathGraphDataOutputStream oups) throws IOException {
		oups.writeInt(size());
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			oups.writeObject(elb);
		}
	}

	public final boolean egalNomValeurOuFonctionOuMesure(String chaine, int indiceMaxiDansListe) {
		CValeurDynamique ptv;

		boolean trouve = false;
		int i = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext() && !trouve && (i <= indiceMaxiDansListe)) {
			i++;
			CElementBase elb = li.next();
			long nat = elb.getNatureCalcul();
			// Modifi verson 4.7.5
			if (((nat & (NatureCalcul.NTouteValeurReelleOuComplexe | NatureCalcul.NTouteFonctionReelle
					| NatureCalcul.NTouteFonctionComplexe | NatureCalcul.NDerivee | NatureCalcul.NTouteSuiteRecurrenteReelle
					| NatureCalcul.NTouteSuiteRecurrenteComplexe)) != 0)
					&& !elb.estElementIntermediaire()) {
				ptv = (CValeurDynamique) elb;
				if (ptv.chaineEgaleANom(chaine))
					trouve = true;
			}
		}
		return trouve;
	}

	/**
	 * Permet de savoir si la liste contient un point qui soit sur la droite Droite1 et la droite Droite2
	 * Si oui renvoie true, sinon renvoie false
	 * */
	private final boolean existePointConfonduPourIntersection(double x, double y, CDroiteAncetre droite1,
			CDroiteAncetre droite2) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.estDeNature(NatureObjet.NToutPoint)) {
				CPointAncetre ptp = (CPointAncetre) elb;
				// Modification version 3.9 : Si l'interection a dj t cre pour un objet intermdiaire de construction
				// on cre de nouveau l'objet
				// if (ptp.existe) {
				if (! ptp.estElementIntermediaire() && ptp.existe) {
					if (Kernel.zero(ptp.x - x) && Kernel.zero(ptp.y - y) && ptp.appartientParDefinition(droite1)
							&& ptp.appartientParDefinition(droite2)) {
						return true;
					}
				}
			}
		}
		return false;
	}

	/**
	 * Permet de savoir si la liste contient un point dont les coordonnes sont
	 * "presque" gales  x et y et qui soit sur la droite Droite1 et le cercle
	 * Cercle1 Si oui renvoirt un pointeur sur ce point, sinon renvoit null
	 * */
	private final CPointAncetre existePointConfonduPourIntersection(double x, double y, CDroiteAncetre droite1,
			CCercle cercle1) {
		ListIterator<CElementBase> li = listIterator();
		CPointAncetre trouve = null;
		while (li.hasNext() && (trouve == null)) {
			CElementBase elb = li.next();
			if (elb.estDeNature(NatureObjet.NToutPoint)) {
				CPointAncetre ptp = (CPointAncetre) elb;
				// Modification version 3.9 : Si l'interection a dj t cre pour un objet intermdiaire de construction
				// on cre de nouveau l'objet
				// if (ptp.existe) {
				if (! ptp.estElementIntermediaire() && ptp.existe) {
					if (Kernel.zero(ptp.x - x) && Kernel.zero(ptp.y - y)
							&& ptp.appartientParDefinition(droite1) && ptp.appartientParDefinition(cercle1)) {
						trouve = ptp;
					}
				}
			}
		}
		return trouve;
	}

	/**
	 * Permet de savoir si la liste contient un point dont les coordonnes sont
	 * "presque" gales  x et y et qui soit sur la droite Droite1 et le cerlce
	 * Cercle1 Si oui renvoirt un pointeur sur ce point, sinon renvoit null
	 * */
	public final CPointAncetre existePointConfonduPourIntersection(double x, double y, CCercle cercle1,
			CCercle cercle2) {
		ListIterator<CElementBase> li = listIterator();
		CPointAncetre trouve = null;
		while (li.hasNext() && (trouve == null)) {
			CElementBase elb = li.next();
			if (elb.estDeNature(NatureObjet.NToutPoint)) {
				CPointAncetre ptp = (CPointAncetre) elb;
				// Modification version 3.9 : Si l'interection a dj t cre pour un objet intermdiaire de construction
				// on cre de nouveau l'objet
				// if (ptp.existe) {
				if (! ptp.estElementIntermediaire() && ptp.existe) {
					if (Kernel.zero(ptp.x - x) && Kernel.zero(ptp.y - y) && ptp.appartientParDefinition(cercle1)
							&& ptp.appartientParDefinition(cercle2)) {
						trouve = ptp;
					}
				}
			}
		}
		return trouve;
	}

	/**
	 * Etudie si la droite droite1 et la droite droite2 sont scants. S'ils le sont
	 * et si un des deux points d'intersection a dj t cr, renvoit un
	 * pointeur sur ce point. Sinon renvoit null. La droite et le cercle existent
	 * avant appel Renvoie true dans DejaCree quand les deux points d'intersection
	 * ont dj t crs et false sinon
	 * */
	public final boolean pointIntersectionDejaCree(CDroiteAncetre droite1, CDroiteAncetre droite2) {
		byte positionRelative;
		// Coordonnes des points d'intersection
		Point2D.Double pointInt1 = new Point2D.Double();
		positionRelative = Kernel.intersection(droite1.point_x, droite1.point_y, droite1.vecteur, droite2.point_x,
				droite2.point_y, droite2.vecteur, pointInt1);
		if (positionRelative != PositionDroites.Paralleles) {
			return existePointConfonduPourIntersection(pointInt1.x, pointInt1.y, droite1, droite2);
		} else
			return false;
	}

	/**
	 * Etudie si la droite Droite et le cercle Cercle sont scants. S'ils le sont
	 * et si un des deux points d'intersection a dj t cr, renvoit un
	 * pointeur sur ce point. Sinon renvoit null. La droite et le cercle existent
	 * avant appel Renvoie true dans DejaCree quand les deux points d'intersection
	 * ont dj t crs et false sinon
	 * */
	public final CPointAncetre pointIntersectionDejaCree(CDroiteAncetre droite1, CCercle cercle1, Bool dejaCree) {
		byte positionRelative;
		// Coordonnes des points d'intersection
		Point2D.Double pointInt1 = new Point2D.Double();
		Point2D.Double pointInt2 = new Point2D.Double();
		dejaCree.setValue(false);
		positionRelative = Kernel.intersectionDroiteCercle(droite1.point_x, droite1.point_y, droite1.vecteur,
				cercle1.centre_x, cercle1.centre_y, cercle1.rayon, pointInt1, pointInt2);
		CPointAncetre ptPoint1 = existePointConfonduPourIntersection(pointInt1.x, pointInt1.y, droite1, cercle1);
		CPointAncetre ptPoint2 = existePointConfonduPourIntersection(pointInt2.x, pointInt2.y, droite1, cercle1);
		if (positionRelative == PositionDroiteCercle.Secants) {
			dejaCree.setValue((ptPoint1 != null) && (ptPoint2 != null));
			if (dejaCree.getValue())
				return null;
			else {
				if (ptPoint1 != null)
					return ptPoint1;
				else {
					if (ptPoint2 != null)
						return ptPoint2;
					else
						return null;
				}
			}
		}
		else {
			if (positionRelative == PositionDroiteCercle.Tangents) {
				dejaCree.setValue(ptPoint1 != null);
				if (dejaCree.getValue()) return ptPoint1;
				else return null;
			}
			else return null;
		}			
	}

	/**
	 * Etudie si la droite Droite et le cercle Cercle sont scants. S'ils le sont
	 * et si un des deux points d'intersection a dj t cr, renvoie un
	 * pointeur sur ce point. Sinon renvoit null. La droite et le cercle existent
	 * avant appel Renvoit true dans DejaCree quand les deux points d'intersection
	 * ont dj t crs et false sinon
	 * */
	public final CPointAncetre pointIntersectionDejaCree(CCercle cercle1, CCercle cercle2, Bool dejaCree) {
		byte positionRelative;
		// Coordonnes des points d'intersection
		Point2D.Double pointInt1 = new Point2D.Double();
		Point2D.Double pointInt2 = new Point2D.Double();
		dejaCree.setValue(false);
		positionRelative = Kernel.intersectionCercleCercle(cercle1.centre_x, cercle1.centre_y, cercle1.rayon,
				cercle2.centre_x, cercle2.centre_y, cercle2.rayon, pointInt1, pointInt2);
		CPointAncetre ptPoint1 = existePointConfonduPourIntersection(pointInt1.x, pointInt1.y, cercle1, cercle2);
		CPointAncetre ptPoint2 = existePointConfonduPourIntersection(pointInt2.x, pointInt2.y, cercle1, cercle2);
		if (positionRelative == PositionCercleCercle.Secants) {
			dejaCree.setValue((ptPoint1 != null) && (ptPoint2 != null));
			if (dejaCree.getValue())
				return null;
			else {
				if (ptPoint1 != null)
					return ptPoint1;
				else {
					if (ptPoint2 != null)
						return ptPoint2;
					else
						return null;
				}
			}
		}
		else {
			if (positionRelative == PositionCercleCercle.Tangents) {
				dejaCree.setValue(ptPoint1 != null);
				if (dejaCree.getValue()) return ptPoint1;
				else return null;
			}
			else return null;
		}
	}

	/**
	 * Fonction renvoyant true si la liste ne contient pas de point ou de droite
	 * autre que element ayant pour nom st
	 * 
	 * @param element
	 *          peut tre null
	 * @param st
	 * @return
	 */
	private final boolean existePointOuDroiteMemeNom(CElementGraphique element, String st) {
		ListIterator<CElementBase> li = listIterator();
		boolean trouve = false;
		while (li.hasNext() && !(trouve)) {
			CElementBase elb = li.next();
			if (! elb.estElementIntermediaire() && ((elb.getNature() & (NatureObjet.NToutPoint + NatureObjet.NDroite)) != 0)) {
				CElementGraphique ptp = (CElementGraphique) elb;
				if (ptp != element) {
					if (ptp.nom.length() != 0) {
						// Modification de la version 2.5
						// if (!NomEl.Compare(St))
						if (ptp.nom.equals(st))
							trouve = true;
					}
				}
			}
		}
		return trouve;
	}

	/**
	 * Fonction gnrant un nom libre pour un point nom nomm
	 * 
	 * @return
	 */
	public final String genereNomPourPoint() {
		String ch = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
		StringBuffer nomGenere = new StringBuffer(64);
		for (int i = 0; i <= Integer.MAX_VALUE; i++) {
			for (int j = 0; j < ch.length(); j++) {
				nomGenere.delete(0, nomGenere.length());
				nomGenere.append(ch.charAt(j));
				if (i != 0)
					nomGenere.append(Integer.toString(j));
				if (!existePointOuDroiteMemeNom(null, nomGenere.toString()))
					return nomGenere.toString();
			}
		}
		return ""; // Pour satisfaire le compilateur
	}

	/**
	 * Fonction renvoyant le nombre d'objets de la liste pour l'utilisateur. Les
	 * trabsformations et bipoints ne sont pas comptes Si traceObjetsInterm est
	 * true les objets intermdiaires sont pris en compte
	 * 
	 * @param traceObjetsinTerm
	 * @return
	 */
	public final int nombreElementsPourHistorique(boolean traceObjetsInTerm) {
		ListIterator<CElementBase> li = listIterator();
		int i = 0;
		while (li.hasNext()) {
			CElementBase elb = li.next();
			boolean estInterm = elb.estElementIntermediaire();
			if (!estInterm || (estInterm && traceObjetsInTerm)) {
				if ((elb.getNature() & (NatureObjet.NTransformation | NatureObjet.NBipoint)) == 0) {
					i++;
				}
			}
		}
		return i;
	}

	
	/**
	 * Ajout version 3.0
	 * Affichage de tous les lments non intermdiaires et non masqus d'indice <=  intder
	 * @param pDC
	 * @param dimfen
	 * @param indDer
	 * @param listeExclue
	 * @param couleurFond
	 * @param transparence
	 */
	public final void afficheNonMasquesJusqueSaufElementsDe(Graphics2D pDC, DimensionFenetre dimfen, int indDer,
			CListeObjets listeExclue, Color couleurFond, float transparence) {
		int ind = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext() && (ind <= indDer)) {
			CElementBase elb = li.next();
			ind++;
			if ((elb.existe) && !listeExclue.contains(elb)) {
				long nature = elb.getNature();
				if ((nature & NatureObjet.NToutObjet) != 0) {
					CElementGraphique elg = (CElementGraphique) elb;
					if (!elb.estElementIntermediaire() && ! elg.masque) {
						elg.affiche(pDC, true, couleurFond, transparence);
						if ((nature & NatureObjet.NObjetNommable) != 0) {
							elg.afficheNom(pDC);
						}
					}
				}
			}
		}
	}
	
	/**
	 * Fonction renvoyant true seulement si la liste contient un objet non
	 * intermdiaire autre que element ayant pour nom st
	 * 
	 * @param element
	 * @param st
	 * @return
	 */
	public final boolean existeMemeNomNonIntermediaire(CElementGraphique element, String st) {
		ListIterator<CElementBase> li = listIterator();
		boolean trouve = false;
		while ((li.hasNext()) && !trouve) {
			CElementBase elb = li.next();
			if (((elb.getNature() & NatureObjet.NToutObjet) != 0) && !elb.estElementIntermediaire()) {
				CElementGraphique elg = (CElementGraphique) elb;
				if (elg != element) {
					if (elg.nom.length() != 0) {
						if (elg.nom.equals(st))
							trouve = true;
					}
				}
			}
		}
		return trouve;
	}

	public final boolean validationNomElement(String chaine) {
		char premierCar, carPrecedent, carEnCours;
		boolean valable;
		// CString ChaineVerification;

		if (chaine.length() == 0)
			return true;
		premierCar = chaine.charAt(0);
		if (!FonteNom.lettre(premierCar))
			return false;
		// Le premier caractre est bien une lettre
		carPrecedent = premierCar;
		valable = true;
		for (int i = 1; i < chaine.length(); i++) {
			carEnCours = chaine.charAt(i);
			// Cas d'une lettre
			if (FonteNom.lettre(carEnCours)) {
				if (FonteNom.chiffre(carPrecedent) || FonteNom.prime(carPrecedent) || FonteNom.lettre(carPrecedent))
					valable = false;
			} else {
				if (FonteNom.prime(carEnCours)) {
					if (FonteNom.chiffre(carPrecedent) || FonteNom.prime(carPrecedent))
						valable = false;
				} else {
					if (!(FonteNom.chiffre(carEnCours)))
						valable = false;
				}
			}
			carPrecedent = carEnCours;
		}
		return valable;
	}

	/**
	 * Fonction rajoute pour pourvoir dplacer le nom d'un objet
	 * 
	 * @param typeCherche
	 * @param point
	 * @param info
	 * @param listeExclusion
	 * @return
	 */
	public final CElementGraphique premierObjetNomProcheDe(Graphics2D dc, long typeCherche, Point point) {
		long nat;
		CElementBase ptel;
		CElementGraphique ptelg;
		double dis;

		// lorsque la liste est vide, aucun objet n'est proche
		if (size() == 0)
			return null;
		// On traite tous les lments de la liste

		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			ptel = li.next();
			nat = ptel.getNature();
			if ((nat & typeCherche) != 0) {
				// Modification par rapport au C++
				ptelg = (CElementGraphique) ptel;
				if (!(ptelg.masque || ptelg.nomMasque)) {
					dis = ptelg.distanceNom(dc, point.x, point.y, true);
					// LPtoDP(Dc,Poindis,1);
					if (dis == 0)
						return ptelg;// On est  l'intrieur de l'affichage du nom de
					// l'objet
				} // if (!Ptel.Masque)
			} // if (nat and type_cherche <>0)
		}// Fin de la boucle de traitement de chacun des lments de la liste
		return null; // Aucun nom d'objet proche
	}

	public final boolean nomIndispensable(CElementGraphique el) {
		CElementBase ptel;
		boolean res = false;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			ptel = li.next();
			res = res || ptel.nomIndispensable(el);
		}
		return res;
	}

	/**
	 * Fonction renvoyant le nombre de variables devant figurer en bas et  droite
	 * de la figure
	 */
	public final int nombreVariablesAvecDialogueAssocie() {
		int res = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNatureCalcul() == NatureCalcul.NVariable) {
				if (((CVariableBornee) elb).dialogueAssocie)
					res++;
			}
		}
		return res;
	}
	
	public final int nombreCalculsModifiablesParMenu() {
		int res = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
      if (!elb.estElementIntermediaire()
      		&& elb.estDeNatureCalcul(NatureCalcul.NCalculOuVariableOuFonctionReelOuComplexe) && elb.modifiableParMenu())
      	res++;
		}
		return res;
	}
	
	/**
	 * Fonction renvoyant le nombre d'images par la transformation transf ne
	 * dpendant pas de exclu
	 * 
	 * @param transf
	 */
	public final int nombreImagesParTransformationNonDepDe(CTransformation transf, CElementBase exclu) {
		ListIterator<CElementBase> li = listIterator();
		int res = 0;
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (!elb.dependDe(exclu)) {
				CElementBase elb2 = elb.antecedentDirect();
				if ((elb2 != elb) && (elb2.getNature() == NatureObjet.NTransformation) && (elb2 == transf))
					res++;
			}
		}
		return res;
	}
	
	public final int nombreImagesParTransformation(CTransformation transf) {
		ListIterator<CElementBase> li = listIterator();
		int res = 0;
		while (li.hasNext()) {
			CElementBase elb = li.next();
			CElementBase elb2 = elb.antecedentDirect();
			if ((elb2 != elb) && (elb2.getNature() == NatureObjet.NTransformation) && (elb2 == transf))
				res++;
		}
		return res;
	}
	
	
	public final int nombreObjetsParNatureCalculNonIntermediaires(long nat) {
		ListIterator<CElementBase> li = listIterator();
		int compt = 0;
		boolean trouve = false;
		while (li.hasNext() && !(trouve)) {
			CElementBase elb = li.next();
			if (!elb.estElementIntermediaire() && elb.estDeNatureCalcul(nat))
				compt++;
		}
		return compt;
	}
	
	public CTransformation transformationConfondueAvec(CTransformation trans) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (!elb.estElementIntermediaire() && (elb.getNature() == NatureObjet.NTransformation)) {
				CTransformation t = (CTransformation) elb;
				if (t.getClass() == trans.getClass()) {
					if (t.confonduAvec(trans)) return t;
				}
			}
		}
		return null;
	}
	

	/**
	 * Fonction utilise quand on zoome ou dzoome sur la figure
	 * Modifi version 3.1.6 : Si un point libre est punais on ne zoome pas ses coordonnes
	 * @param xcentre
	 * @param ycentre
	 * @param rapport
	 */
	public final void zoom(double xcentre, double ycentre, double rapport) {
		CPointAncetre origineUnite = null;
		if (pointeurLongueurUnite != null) {
			CPointAncetre pt1 = pointeurLongueurUnite.point1;
			if (pt1.getNature() == NatureObjet.NPointBase)
				origineUnite = pt1;
		}
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if ((elb.getNature() == NatureObjet.NPointBase) && (elb != origineUnite)) {
				CPointBase ptb = (CPointBase) elb;
				if (ptb.estLibre()) ptb.zoom(xcentre, ycentre, rapport);
				
			}
			else {
				if (elb.getNature() == NatureObjet.NDroite) {
					if (elb.getClass() == CDroiteDirectionFixe.class)
						((CDroiteDirectionFixe) elb).zoom(rapport);
				}
			}
		}
	}

	/**
	 * Fonction reconstruisant toutes les chanes des calculs et des fonctions.
	 * Utilis quand on charge un document dont la langue n'est pas la langue
	 * active.
	 */
	public void reconstruitChainesCalcul() {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if ((elb.getNatureCalcul() & NatureCalcul.NCalculOuFonctionNonConstante) != 0)
				((CCalcul) elb).reconstruitChaineCalcul();
		}
	}

	public boolean sontParDefinitionSurParallele(CPointAncetre pt1, CPointAncetre pt2, CDroiteAncetre droite) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if ((elb.getNature() & NatureObjet.NTouteDroiteSaufVecteur) != 0) {
				CDroiteAncetre da = (CDroiteAncetre) elb;
				if (pt1.appartientParDefinition(da) && pt2.appartientParDefinition(da)
						&& (da.estParalleleParDefinition(droite) || droite.estParalleleParDefinition(da))) return true;
			}
		}
		return false;
	}
	
	public final void recreeListesCopiesLieuxObjetsDependantDe(CLieuObjetAncetre lieu) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if ((elb.getNature() == NatureObjet.NLieuObjet) && (elb != lieu) && elb.dependDe(lieu)) {
				((CLieuObjetAncetre)elb).recreeListeCopieObjets();
			}
		}
	}
	
	/**
	 * Fonction renvoyant true seulement si nom est valide pour un nouveau nom de calcul ou de fonction
	 * N'affiche pas de message en cas d'erreur
	 * les paramtres formels de fonctions sont autoriss
	 * @param chNom
	 * @return
	 */
	public final boolean validationNomCalculSansMessage(String chNom) {
    if ((chNom.length() == 0) || !EditeurNomCalcul.validationNomVariableOuCalcul(chNom)) {
      return false;
    }
    if (chNom.equals("i")) {
      return false;
    }
    if (CCalculBase.egalNomFonctionReelleOuComplexePredefinie(chNom)) {
      return false;	    	
    }		
    if (egalNomValeurOuFonctionOuMesure(chNom, size()-1)) {
      return false;
    }
    return true;
	}
	
	public final void initialiseNombreIterations() {
		nombreIterations = 0;
	}
	
	/**
	 * Ajout version 3.4.2 pour optimisation
	 * Rinitialize le membre elementTestePourDependDe de chaque objet  null
	 * Doit tre appel lorsque les liens entre objets peuvent avoir chang
	 * Cette opimisation a t rajoute pour optimiser le temps de chargement et de cration
	 * des lieux de points et lieux d'objets (on pouvait opour certaines figures
	 * avoir une latence de plusieurs minutes)
	 */
	public final void initialiseDependances() {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			elb.initialisePourDependance();
		}
	}
	
	
}