/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.*;
import java.io.*;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CMarqueSegment extends CElementLigne {
	public int motif; // Le motif
	CSegment segmentAssocie;
	// Pointeurs sur des points crs de faon interne comme extmits
	// des segments crant la marque
	CPointBase origine1, extremite1, origine2, extremite2, origine3, extremite3;
	// Pointeurs sur des segments utiliss de faon interne pour tracer les marques
	CSegment segment1, segment2, segment3;
  // Comme la mthode de trace utilise des segments internes, ces segments doivent tre
  // positionns dans setClone(). Il faut donc mmoriser dans positionne(infoRandom, dimfen)
  // les valeurs de infoRandom et dimfen.
  private boolean infoR;
  private DimensionFenetre dimf;

  /** Creates new CMarqueSegment */
  public CMarqueSegment() {
  }
  
  // Constructeur pour clonage
  public CMarqueSegment(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit, int styleInit, int motifInit, CSegment segmentAssocieInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
    motif = motifInit;
    segmentAssocie = segmentAssocieInit;
    origine1 = new CPointBase();
    extremite1 = new CPointBase();
    origine2 = new CPointBase();
    extremite2 = new CPointBase();
    origine3 = new CPointBase();
    extremite3 = new CPointBase();
    segment1 = new CSegment(liste, origine1, extremite1);
    segment2 = new CSegment(liste, origine2, extremite2);
    segment3 = new CSegment(liste, origine3, extremite3);
  }  

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(segmentAssocie);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMarqueSegment(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, motif, (CSegment)(listeCible.get(ind1, CSegment.class)));
    return ptelb;
  }

  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    origine1.placeEn(((CMarqueSegment)ptel).origine1);
    origine2.placeEn(((CMarqueSegment)ptel).origine2);
    origine3.placeEn(((CMarqueSegment)ptel).origine3);
    extremite1.placeEn(((CMarqueSegment)ptel).extremite1);
    extremite2.placeEn(((CMarqueSegment)ptel).extremite2);
    extremite3.placeEn(((CMarqueSegment)ptel).extremite3);
    segment1.donneStyle(((CMarqueSegment)ptel).segment1.style);
    segment2.donneStyle(((CMarqueSegment)ptel).segment2.style);
    segment3.donneStyle(((CMarqueSegment)ptel).segment3.style);
    segment1.donneCouleur(((CMarqueSegment)ptel).segment1.couleur);
    segment2.donneCouleur(((CMarqueSegment)ptel).segment2.couleur);
    segment3.donneCouleur(((CMarqueSegment)ptel).segment3.couleur);
    // Les segments n'ont t positionns que dans la liste principale, pas dans
    // la liste clone
    if (((CMarqueSegment)ptel).segment1.existe) segment1.positionne(infoR, dimf);
      else segment1.existe = false;
    if (((CMarqueSegment)ptel).segment2.existe) segment2.positionne(infoR, dimf);
      else segment2.existe = false;
    if (((CMarqueSegment)ptel).segment3.existe) segment3.positionne(infoR, dimf);
      else segment3.existe = false;
  }
  
 public void donneMotif(int newMotif) {
	 motif = newMotif;
 }
  @Override
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || segmentAssocie.dependDe(p);
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || segmentAssocie.dependDePourBoucle(p);
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double xmil,ymil,nor,xext1,yext1,xext2,yext2;

    if (!segmentAssocie.existe)
    {
      existe = false;
      return;
    }
    infoR = infoRandom;
    dimf = dimfen;
    segment1.donneStyle(style);
    segment2.donneStyle(style);
    segment3.donneStyle(style);
    segment1.donneCouleur(couleur);
    segment2.donneCouleur(couleur);
    segment3.donneCouleur(couleur);
    xmil = (segmentAssocie.point1.x + segmentAssocie.point2.x)/2;
    ymil = (segmentAssocie.point1.y + segmentAssocie.point2.y)/2;
    VecteurDirecteur u = new VecteurDirecteur(xmil, ymil, segmentAssocie.point2.x, segmentAssocie.point2.y);
    boolean basseDefinition = listeProprietaire.basseDefinition;
    nor = u.norme();
    u.x = u.x/nor;
    u.y = u.y/nor;
    VecteurDirecteur v = new VecteurDirecteur();
    u.tourne(Math.PI/2,v);
    v.x = v.x*5;
    v.y = v.y*5;
    if (!basseDefinition) {
    	u.x *= 10;
    	u.y *= 10;
    	v.x *= 10;
    	v.y *= 10;
    }

    switch (motif)
    {
      case MotifMarqueSegment.MarqueSimple:

        xext1 = xmil + v.x;
        yext1 = ymil + v.y;
        xext2 = xmil - v.x;
        yext2 = ymil - v.y;
        origine1.placeEn(xext1,yext1);
        extremite1.placeEn(xext2,yext2);
        segment1.positionne(infoRandom, dimfen);
        segment2.existe = false;
        segment3.existe = false;
        break;

      case MotifMarqueSegment.MarqueDouble:
        xext1 = xmil + v.x + 2*u.x;  //on rajoute 2 pixels par rapport au milieu
        yext1 = ymil + v.y + 2*u.y;
        xext2 = xmil - v.x + 2*u.x;
        yext2 = ymil - v.y + 2*u.y;
        origine1.placeEn(xext1,yext1);
        extremite1.placeEn(xext2,yext2);
        segment1.positionne(infoRandom, dimfen);
        xext1 = xmil + v.x - 2*u.x;  //on retranche 3 pixels par rapport au milieu
        yext1 = ymil + v.y - 2*u.y;
        xext2 = xmil - v.x - 2*u.x;
        yext2 = ymil - v.y - 2*u.y;
        origine2.placeEn(xext1,yext1);
        extremite2.placeEn(xext2,yext2);
        segment2.positionne(infoRandom, dimfen);
        segment3.existe = false;
        break;

      case MotifMarqueSegment.MarqueTriple:
        xext1 = xmil + v.x;
        yext1 = ymil + v.y;
        xext2 = xmil - v.x;
        yext2 = ymil - v.y;
        origine1.placeEn(xext1,yext1);
        extremite1.placeEn(xext2,yext2);
        segment1.positionne(infoRandom, dimfen);
        xext1 += 4*u.x;
        yext1 += 4*u.y;
        xext2 += 4*u.x;
        yext2 += 4*u.y;
        origine2.placeEn(xext1,yext1);
        extremite2.placeEn(xext2,yext2);
        segment2.positionne(infoRandom, dimfen);
        xext1 -= 8*u.x;
        yext1 -= 8*u.y;
        xext2 -= 8*u.x;
        yext2 -= 8*u.y;
        origine3.placeEn(xext1,yext1);
        extremite3.placeEn(xext2,yext2);
        segment3.positionne(infoRandom, dimfen);
        break;

      case MotifMarqueSegment.MarqueCroix:
        VecteurDirecteur w = new VecteurDirecteur();
        u.tourne(Math.PI/4,w);
        w.x = w.x * 5;
        w.y = w.y * 5;
        xext1 = xmil + w.x;
        yext1 = ymil + w.y;
        xext2 = xmil - w.x;
        yext2 = ymil - w.y;
        origine1.placeEn(xext1,yext1);
        extremite1.placeEn(xext2,yext2);
        segment1.positionne(infoRandom, dimfen);
        u.tourne(-Math.PI/4,w);
        w.x = w.x * 5;
        w.y = w.y * 5;
        xext1 = xmil + w.x;
        yext1 = ymil + w.y;
        xext2 = xmil - w.x;
        yext2 = ymil - w.y;
        origine2.placeEn(xext1,yext1);
        extremite2.placeEn(xext2,yext2);
        segment2.positionne(infoRandom, dimfen);
        segment3.existe = false;
        break;
     }
    existe = true;
  }

  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    if (segment1.existe) segment1.trace(pDC, couleurFond, transparence);
    if (segment2.existe) segment2.trace(pDC, couleurFond, transparence);
    if (segment3.existe) segment3.trace(pDC, couleurFond, transparence);
  }
 
  @Override
  public boolean confonduAvec(CElementBase p) {
    CMarqueSegment pt;
    if (p.getClass() == this.getClass()) {
      pt = (CMarqueSegment) p;
      return (motif == pt.motif) && (segmentAssocie == pt.segmentAssocie);
    }
    else return false;
  }  
  
  public double distancePoint(double xp, double yp, boolean masquage) {
  	double dist;

  	if ((!existe) || (masquage && masque)) return -1;
  	else {
  		if (segment1.existe) {
  			dist = segment1.distancePoint(xp,yp,masquage);
  			if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
  		}
  		if (segment2.existe) {
  			dist = segment2.distancePoint(xp,yp,masquage);
  			if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
  		}
  		if (segment3.existe) {
  			dist = segment3.distancePoint(xp,yp,masquage);
  			if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
  		}
  		return -1;
  	}
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NMarqueSegment;
  }

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
     super.chargeDepuisFlux(inps, list);
     origine1 = new CPointBase();
     extremite1 = new CPointBase();
     origine2 = new CPointBase();
     extremite2 = new CPointBase();
     origine3 = new CPointBase();
     extremite3 = new CPointBase();
     segment1 = new CSegment(list, origine1, extremite1);
     segment2 = new CSegment(list, origine2, extremite2);
     segment3 = new CSegment(list, origine3, extremite3);
     motif = inps.readByte();
     int ind1 = inps.readCInt();
     segmentAssocie = (CSegment) (list.get(ind1,CSegment.class));
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    origine1 = new CPointBase();
    extremite1 = new CPointBase();
    origine2 = new CPointBase();
    extremite2 = new CPointBase();
    origine3 = new CPointBase();
    extremite3 = new CPointBase();
    segment1 = new CSegment(list, origine1, extremite1);
    segment2 = new CSegment(list, origine2, extremite2);
    segment3 = new CSegment(list, origine3, extremite3);
    motif = inps.readByte();
    int ind1 = inps.readInt();
    segmentAssocie = (CSegment) (list.get(ind1,CSegment.class));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeByte(motif);
    int ind1 = list.indexOf(segmentAssocie);
    oups.writeInt(ind1);
  }  
}
