/*
 * CMesureAffixe.java
 *
 * Created on 28 janvier 2007, 18:22
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package objets;

import java.io.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

/**
 * Classe ajoute pour la version 2.0
 * @author Yves
 */
public class CMesureAffixe extends CValeurDynamique {
  Complexe affixe = new Complexe();
  CRepere repereAssocie;
  CPointAncetre pointMesure;
  
  /** Creates a new instance of CMesureAffixe */
  public CMesureAffixe() {
  }
  // Constructeur pour clonage
  public CMesureAffixe(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CRepere repereAssocieInit, CPointAncetre pointMesureInit) {
    super(liste, impProtoInit, estElementFinalInit);
    repereAssocie = repereAssocieInit;
    pointMesure = pointMesureInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(repereAssocie);
    int ind2 = listeSource.indexOf(pointMesure);
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMesureAffixe(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, (CRepere)(listeCible.get(ind1, CRepere.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)));
    return ptelb;    
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NMesureAffixe;
  }
  
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || repereAssocie.dependDe(p) || pointMesure.dependDe(p);
  }

  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || repereAssocie.dependDePourBoucle(p) || pointMesure.dependDePourBoucle(p);
  }

  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = repereAssocie.existe && pointMesure.existe;
    if (!existe) return;
    // L'orientation sur l'cran est l'inverse de celle du repre
		// choisi.Il faut donc permuter les rles de u et v}
    VecteurDirecteur u = new VecteurDirecteur(repereAssocie.O,repereAssocie.I);
    VecteurDirecteur v = new VecteurDirecteur(repereAssocie.O,repereAssocie.J);
    // Calcul des	abscisses et ordonnes de l'origine relle du repre dans le repre de base
    double xno = repereAssocie.O.x - u.x*repereAssocie.abscisseOrigine.rendValeur(infoRandom)-
            v.x*repereAssocie.ordonneeOrigine.rendValeur(infoRandom);
    double yno = repereAssocie.O.y - u.y*repereAssocie.abscisseOrigine.rendValeur(infoRandom)-
            v.y*repereAssocie.ordonneeOrigine.rendValeur(infoRandom);
    // Calcul de l'abscisse et de l'ordonne par rsolution d'un systme linaire
    double det = u.x*v.y-v.x*u.y;
    affixe.x = (v.y*(pointMesure.x-xno)-v.x*(pointMesure.y-yno))/det;
    affixe.y = (u.x*(pointMesure.y-yno)-u.y*(pointMesure.x-xno))/det;  
  }
  
  // Inutilis mais pour le compilateur
  @Override
  public double rendValeur(boolean infoRandom) {
    return 0;
  }
  public void rendValeurComplexe(boolean infoRandom, Complexe zRes) {
    zRes.set(affixe.x, affixe.y);
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CMesureAffixe pt;
    if (p.getClass() == this.getClass()) {
      pt = (CMesureAffixe) p;
      return (repereAssocie == pt.repereAssocie) && (pointMesure == pt.pointMesure);
    }
    else return false;
  }
  
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
     super.chargeDepuisFlux(inps, list);
     int ind1 = inps.readCInt();
     int ind2 = inps.readCInt();
     repereAssocie = (CRepere) (list.get(ind1,CRepere.class));
     pointMesure = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }
  /* Lecture depuis dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    repereAssocie = (CRepere) (list.get(ind1,CRepere.class));
    pointMesure = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(repereAssocie);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(pointMesure);
    oups.writeInt(ind2);
  }  
}
