/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;


// Contrairement  la version C++, il est inutile de dfinir getNature pour chaque objet
// car cela ne sert que pour les objets dsignales, les macros
// La nature NPoint qui est la nature du point construit sera la nature des points dfinie
// dans CPointAncetre. Il ne sera redfini que pour les points mobiles (pojnts libres et points lis).
import java.awt.*;
import java.io.*;
import java.awt.geom.*;

import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

// Modification pour la version 2.5 : la classe n'est plus abstract car il
// faut pouvoir l'instancier dans CListeObjets.get(int indice, Class cla)
/**
 * Classe anctre de tous les points
 */
public class CPointAncetre extends CElementGraphique {
  public double x, y;
  public int motif;
  public boolean horsEcran;
  private boolean dansFenetre; // Rajout version 4.6.4
  // Modification pour la version 4.4. Le numero de version de document passe  8 et chaque point peut tre marqu ou non pour la trace
  public boolean marquePourTrace;

  /** Creates new CPointAncetre */
  protected CPointAncetre() {
  }
  
  protected CPointAncetre(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
    boolean masqueInit, String nomInit, int indiceTailleNomInit, int motifInit, boolean marquePourTraceInit) {
      super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
        dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit);
      motif = motifInit;
      marquePourTrace = marquePourTraceInit;
  }

  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    x = ((CPointAncetre)ptel).x;
    y = ((CPointAncetre)ptel).y;
    horsEcran = ((CPointAncetre)ptel).horsEcran;
    dansFenetre = ((CPointAncetre)ptel).dansFenetre; // Ajout version 4.6.4
    marquePourTrace = ((CPointAncetre)ptel).marquePourTrace;
  }

  public boolean testToile() {
    return Kernel.testToile(x, y);
  }
  
  /**
   * Fonction testant si le point est dans la fentre de dimensions dimf
   * @param dimf
   * @return
   */
  @Override
  public boolean dansFenetre(DimensionFenetre dimf) {
  	return testFenetre(dimf, x, y);
  }
  
  // GetNature ne sera redfini que pour les points libres et points lis
  // Grosse diffrence par rapport au C++
  @Override
  public long getNature() {
    return NatureObjet.NPoint;
  }
  /**
   * Fonction testant si le point est dans la fentre de dimensions dimf
   * @param dimf
   * @return
   */
  boolean testFenetre(DimensionFenetre dimfen, double x, double y)
  {
     return dimfen.dansFenetre(x, y);
  }

  @Override
  public void affiche(Graphics2D pDC, boolean masquage, Color couleurFond, float transparence) {
  	// Modifi version 4.6.4
    // if (!(masquage && masque) && !horsEcran) trace(pDC, couleurFond, transparence);
  	if (!(masquage && masque) && dansFenetre) trace(pDC, couleurFond, transparence);
  }
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence)
  {
    int xint,yint;

    if (horsEcran) return;
    xint = (int) Math.round(x);
    yint = (int) Math.round(y);

    // Certains objets servent de faon interne et ne sont pas associs  une liste
    Stroke stroke = Pinceau.CreePinceau(StyleTrait.traitFin);
    // Affichage d'une toile si le point est masqu
    if (masque) {
      pDC.setStroke(stroke);
      pDC.setColor(couleur);
      pDC.drawLine(xint-3, yint, xint+3, yint);
      pDC.drawLine(xint-3, yint-3, xint+3, yint+3);        
      pDC.drawLine(xint-3, yint+3, xint+3, yint-3);
      pDC.drawLine(xint, yint-3, xint, yint+3);
    }
    else {
	    switch (motif)
	    {
	      case MotifPoint.pave:
	          pDC.setStroke(stroke);
	          pDC.setColor(couleur);
	          pDC.fillRect(xint-2,yint-2,4,4);
	          break;
	      case MotifPoint.rond:
	          pDC.setStroke(stroke);
	          pDC.setColor(couleur);
	          pDC.drawOval(xint-3,yint-3,6,6);
	          pDC.fillOval(xint-3,yint-3,6,6);
	          break;
	      case MotifPoint.croix:
	          pDC.setStroke(stroke);
	          pDC.setColor(couleur);
	          pDC.drawLine(xint-3, yint, xint+2, yint);
	          pDC.drawLine(xint-3, yint-1, xint+2, yint-1);        
	
	          pDC.drawLine(xint-1, yint-3, xint-1, yint+2);
	          pDC.drawLine(xint, yint-3, xint, yint+2);
	
	          break;
	      case MotifPoint.multi:
	          pDC.setStroke(stroke);
	          pDC.setColor(couleur);
	          pDC.drawLine(xint-3, yint-2, xint+1, yint+2);
	          pDC.drawLine(xint-2, yint-2, xint+2, yint+2);
	          pDC.drawLine(xint-3, yint+2, xint+1, yint-2);
	          pDC.drawLine(xint-2, yint+2, xint+2, yint-2);
	          break;
	      case MotifPoint.petitRond:
	          pDC.setStroke(stroke);
	          pDC.setColor(couleur);
	          pDC.fillOval(xint-2,yint-2,4,4);
	          pDC.drawOval(xint-2,yint-2,4,4);
	          break;
	      case MotifPoint.losange:
	          int[] abscisses = new int[4];
	          int[] ordonnees = new int[4];
	          pDC.setStroke(stroke);
	          pDC.setColor(couleur);
	          abscisses[0] = xint;
	          // 1 pixel de plus par rapport au C++ car seul l'intrieur du
	          //est rempli
	          ordonnees[0] = yint-3;
	          abscisses[1] = xint - 3;
	          ordonnees[1] = yint;
	          abscisses[2] = xint;
	          ordonnees[2] = yint + 3;
	          abscisses[3] = xint + 3;
	          ordonnees[3] = yint;
	          pDC.fillPolygon(abscisses, ordonnees, 4);
	          break;
	          // A revoir pour la version java
	     case MotifPoint.pixel:
	          pDC.setStroke(stroke);
	          pDC.setColor(couleur);
	          pDC.drawLine(xint-1,yint,xint+1,yint);
	          //pDC.drawLine(xint-1,yint,xint-1,yint);
	          pDC.drawLine(xint,yint-1,xint,yint+1);
	          //pDC.drawLine(xint,yint+1,xint,yint+1);
	          break;
	
	        // Motif qui ne sera utilis que pour encadrer un barycentre
	        // lorsqu'on doit en dsigner un barycentre pour le modifier
	          // A revoir. Peut tre supprim pour la version java
	    } // switch
    }
  }
  
  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
    if (!(existe) || (masquage && masque))
    return -1;
    else
    {
      VecteurDirecteur u = new VecteurDirecteur(x,y,xp,yp);
      return u.norme();
    }
  }

  @Override
  public void afficheNom(Graphics2D pDC) {
    if (existe && !horsEcran) {
      super.afficheNom(pDC);
    }
  }

  public void positionneNom()
  {
    placeNom(x,y);
  }
  
  @Override  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    horsEcran = !(testToile());
    dansFenetre = testFenetre(dimfen, x, y); // Ajout version 4.6.4
    positionneNom();
  }
  public void donneMotif(int mot)
  {
    motif = mot;
  }
  public void placeEn(double xn, double yn) {
    existe = true;
    x = xn;
    y = yn;
    placeNom(xn, yn);
    // Diffrence par rapport au C++ : on ne peut pas appeler positionne de CPointAncetre
    horsEcran = !(testToile());
    positionneNom();
  }

  // Java : la procdure suivante a t rajoute en java
  public void placeEn(CPointAncetre point) {
    existe = point.existe;
    x = point.x;
    y = point.y;
    placeNom(x, y);
    // Diffrence par rapport au C++ : on ne peut pas appeler positionne de CPointAncetre
    horsEcran = !(testToile());
    positionneNom();
  }

  // La procdure suivante test si lengthdplacement du point correspondant  la position
  // (xtest, ytest)  de la souris est possible et si oui renvoyant dans pointr les
  // nouvelles coordonnes et dans pointAbs.x l'abscisse pour un point li

  public boolean testDeplacement(DimensionFenetre dimfen, double xtest, double ytest,
        Point2D.Double pointr, Doub abscr)
  {
    pointr.x = xtest;
    pointr.y = ytest;
    return testFenetre(dimfen, xtest, ytest);
  }
  
  /**
   * Fonction renvoyant true si le point est sur la droite (ou descendant de droite)
   * @param droite
   * @return
   */
  public boolean surDroite(CDroiteAncetre droite) {
    if (!(existe)) return false;
    VecteurDirecteur vec = new VecteurDirecteur();
    vec.x = x - droite.point_x;
    vec.y = y - droite.point_y;
    return (Kernel.colineaires(droite.vecteur,vec) && droite.appartientA(x,y));
  }
    
  public boolean egal(CPointAncetre pt) {
    return ((this.x == pt.x) && (this.y == pt.y));
  }

  public boolean presqueEgal(CPointAncetre A) {
   return (Kernel.zero(x - A.x) && (Kernel.zero(y - A.y)));
  }

  public final void tourne(double angle, VecteurDirecteur v)
  {
    double kc,ks;

    kc = Math.cos(angle);
    ks = Math.sin(angle);
    v.x = kc*x + ks*y;
    v.y = kc*y - ks*x;
  }
    
  @Override
  public boolean coincideAvec(CElementBase p) {
    if ((p.getNature() & NatureObjet.NToutPoint) == 0) return false;
    CPointAncetre pt = (CPointAncetre) p;
    return (Kernel.zero(pt.x-x) && Kernel.zero(pt.y-y));
  }

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    motif = inps.readByte();
    marquePourTrace = false;
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    motif = inps.readByte();
    if (listeProprietaire.numeroVersion < 8) marquePourTrace = false;
    else marquePourTrace = inps.readBoolean();
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeByte(motif);
    oups.writeBoolean(marquePourTrace);
  }
  
  /** Fonction ajoute pour la version 2.1 en vue d'amliorer l'outil de cration d'intersection
   * Renvoie true si le point est sur la droite par dfinition
   * A redfinir pour les descendants concerns
   * */
  public boolean appartientParDefinition(CDroiteAncetre droite)
  {
    return droite.contientParDefinition(this);
  }
  /** Fonction ajoute pour la version 2.1 en vue d'amliorer l'outil de cration d'intersection
   * Renvoie true si le point est sur la droite par dfinition
   * A redfinir pour les descendants concerns
   * */
  public boolean appartientParDefinition(CCercle cercle)
  {
    return cercle.contientParDefinition(this);
  }
      
  @Override
  public String chaineDesignation() {
    return getString("designationPoint");
  }
  
  /** A partir de la version 1.2.0, il est souvent ncessaire de connatre si un point est un point image par une transformation
   * Ne pouvant pas sans problme rajouter un nouveau type s'objet graphique, on cre cette fonction redfinie seulement
   * pour la classe CPointImage
   * @return
   */
  public boolean estPointImage() {
  	return false;
  }
}

