/*
 * CPointAncetre.java
 *
 * Created on 25 juin 2004, 19:08
 */

/**
 *
 * @author  Yves
 * @version 
 */

package objets;


// Contrairement  la version C++, il est inutile de dfinir getNature pour chaque objet
// car cela ne sert que pour les objets dsignales, les macros
// La nature NPoint qui est la nature du point construit sera la nature des points dfinie
// dans CPointAncetre. Il ne sera redfini que pour les points mobiles (pojnts libres et points lis).
import java.awt.*;
import java.io.*;
import java.awt.geom.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

// Modification pour la version 2.5 : la classe n'est plus abstract car il
// faut pouvoir l'instancier dans CListeObjets.get(int indice, Class cla)
public class CPointAncetre extends CElementGraphique {
  public double x, y;
  public int motif;
  public boolean horsEcran;

  /** Creates new CPointAncetre */
  protected CPointAncetre() {
  }
  
  protected CPointAncetre(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
    boolean masqueInit, String nomInit, int indiceTailleNomInit, int motifInit) {
      super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
        dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit);
      motif = motifInit;
  }

  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    x = ((CPointAncetre)ptel).x;
    y = ((CPointAncetre)ptel).y;
    horsEcran = ((CPointAncetre)ptel).horsEcran;
  }

  public boolean testToile() {
    return Kernel.testToile(x, y);
  }
  
  /**
   * Fonction testant si le point est dans la fentre de dimensions dimf
   * @param dimf
   * @return
   */
  @Override
  public boolean dansFenetre(DimensionFenetre dimf) {
  	return testFenetre(dimf, x, y);
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NPoint;
  }
  /**
   * Fonction testant si le point est dans la fentre de dimensions dimf
   * @param dimf
   * @return
   */
  boolean testFenetre(DimensionFenetre dimfen, double x, double y)
  {
     return dimfen.dansFenetre(x, y);
  }

  @Override
  public void affiche(Graphics2D pDC, boolean masquage, Color couleurFond, float transparence) {
    if (!(masquage && masque) && testToile()) trace(pDC, couleurFond, transparence);
  }
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence)
  {
    int xint,yint;

    if (horsEcran) return;
    xint = (int) Math.round(x);
    yint = (int) Math.round(y);

    // Certains objets servent de faon interne et ne sont pas associs  une liste
    boolean basseDefinition = listeProprietaire.basseDefinition;
    Stroke stroke = Pinceau.CreePinceau(StyleTrait.traitFin);
    if (basseDefinition) {
	    // Affichage d'une toile si le point est masqu
	    if (masque) {
	      pDC.setStroke(stroke);
	      pDC.setColor(couleur);
	      pDC.drawLine(xint-3, yint, xint+3, yint);
	      pDC.drawLine(xint-3, yint-3, xint+3, yint+3);        
	      pDC.drawLine(xint-3, yint+3, xint+3, yint-3);
	      pDC.drawLine(xint, yint-3, xint, yint+3);
	    }
	    else {
		    switch (motif)
		    {
		      case MotifPoint.pave:
		          pDC.setStroke(stroke);
		          pDC.setColor(couleur);
		          pDC.fillRect(xint-2,yint-2,4,4);
		          break;
		      case MotifPoint.rond:
		          pDC.setStroke(stroke);
		          pDC.setColor(couleur);
		          pDC.drawOval(xint-3,yint-3,6,6);
		          pDC.fillOval(xint-3,yint-3,6,6);
		          break;
		      case MotifPoint.croix:
		          pDC.setStroke(stroke);
		          pDC.setColor(couleur);
		          pDC.drawLine(xint-3, yint, xint+2, yint);
		          pDC.drawLine(xint-3, yint-1, xint+2, yint-1);        
		
		          pDC.drawLine(xint-1, yint-3, xint-1, yint+2);
		          pDC.drawLine(xint, yint-3, xint, yint+2);
		
		          break;
		      case MotifPoint.multi:
		          pDC.setStroke(stroke);
		          pDC.setColor(couleur);
		          pDC.drawLine(xint-3, yint-2, xint+1, yint+2);
		          pDC.drawLine(xint-2, yint-2, xint+2, yint+2);
		          pDC.drawLine(xint-3, yint+2, xint+1, yint-2);
		          pDC.drawLine(xint-2, yint+2, xint+2, yint-2);
		          break;
		      case MotifPoint.petitRond:
		          pDC.setStroke(stroke);
		          pDC.setColor(couleur);
		          pDC.fillOval(xint-2,yint-2,4,4);
		          pDC.drawOval(xint-2,yint-2,4,4);
		          break;
		      case MotifPoint.losange:
		          int[] abscisses = new int[4];
		          int[] ordonnees = new int[4];
		          pDC.setStroke(stroke);
		          pDC.setColor(couleur);
		          abscisses[0] = xint;
		          // 1 pixel de plus par rapport au C++ car seul l'intrieur du
		          //est rempli
		          ordonnees[0] = yint-3;
		          abscisses[1] = xint - 3;
		          ordonnees[1] = yint;
		          abscisses[2] = xint;
		          ordonnees[2] = yint + 3;
		          abscisses[3] = xint + 3;
		          ordonnees[3] = yint;
		          pDC.fillPolygon(abscisses, ordonnees, 4);
		          break;
		          // A revoir pour la version java
		     case MotifPoint.pixel:
		          pDC.setStroke(stroke);
		          pDC.setColor(couleur);
		          pDC.drawLine(xint-1,yint,xint+1,yint);
		          //pDC.drawLine(xint-1,yint,xint-1,yint);
		          pDC.drawLine(xint,yint-1,xint,yint+1);
		          //pDC.drawLine(xint,yint+1,xint,yint+1);
		          break;
		
		        // Motif qui ne sera utilis que pour encadrer un barycentre
		        // lorsqu'on doit en dsigner un barycentre pour le modifier
		          // A revoir. Peut tre supprim pour la version java
		    } // switch
	    }
    }
    else {
      if (masque) {
        pDC.setStroke(stroke);
        pDC.setColor(couleur);
        pDC.drawLine(xint-30, yint, xint+30, yint);
        pDC.drawLine(xint-30, yint-30, xint+30, yint+30);        
        pDC.drawLine(xint-30, yint+30, xint+30, yint-30);
        pDC.drawLine(xint, yint-30, xint, yint+30);
      }
      else {
  		    switch (motif)
  		    {
  		      case MotifPoint.pave:
  		          pDC.setStroke(stroke);
  		          pDC.setColor(couleur);
  		          pDC.fillRect(xint-20,yint-20,40,40);
  		          break;
  		      case MotifPoint.rond:
  		          pDC.setStroke(stroke);
  		          pDC.setColor(couleur);
  		          pDC.drawOval(xint-30,yint-30,60,60);
  		          pDC.fillOval(xint-30,yint-30,60,60);
  		          break;
  		      case MotifPoint.croix:
  		          pDC.setStroke(stroke);
  		          pDC.setColor(couleur);
  		          pDC.drawLine(xint-30, yint, xint+20, yint);
  		          pDC.drawLine(xint-30, yint-10, xint+20, yint-10);        
  		          pDC.drawLine(xint-10, yint-30, xint-10, yint+20);
  		          pDC.drawLine(xint, yint-30, xint, yint+20);
  		          break;
  		      case MotifPoint.multi:
  		          pDC.setStroke(stroke);
  		          pDC.setColor(couleur);
  		          pDC.drawLine(xint-30, yint-20, xint+10, yint+20);
  		          pDC.drawLine(xint-20, yint-20, xint+20, yint+20);
  		          pDC.drawLine(xint-30, yint+20, xint+10, yint-20);
  		          pDC.drawLine(xint-20, yint+20, xint+20, yint-20);
  		          break;
  		      case MotifPoint.petitRond:
  		          pDC.setStroke(stroke);
  		          pDC.setColor(couleur);
  		          pDC.fillOval(xint-20,yint-20,40,40);
  		          pDC.drawOval(xint-20,yint-20,40,40);
  		          break;
  		      case MotifPoint.losange:
  		          int[] abscisses = new int[4];
  		          int[] ordonnees = new int[4];
  		          pDC.setStroke(stroke);
  		          pDC.setColor(couleur);
  		          abscisses[0] = xint;
  		          // 1 pixel de plus par rapport au C++ car seul l'intrieur du
  		          //est rempli
  		          ordonnees[0] = yint-30;
  		          abscisses[1] = xint - 30;
  		          ordonnees[1] = yint;
  		          abscisses[2] = xint;
  		          ordonnees[2] = yint + 30;
  		          abscisses[3] = xint + 30;
  		          ordonnees[3] = yint;
  		          pDC.fillPolygon(abscisses, ordonnees, 4);
  		          break;
  		          // A revoir pour la version java
  		     case MotifPoint.pixel:
  		          pDC.setStroke(stroke);
  		          pDC.setColor(couleur);
  		          pDC.drawLine(xint-10,yint,xint+10,yint);
  		          pDC.drawLine(xint,yint-10,xint,yint+10);
  		          break;
  		
  		        // Motif qui ne sera utilis que pour encadrer un barycentre
  		        // lorsqu'on doit en dsigner un barycentre pour le modifier
  		          // A revoir. Peut tre supprim pour la version java
  		    } // switch
	      }	    	
	    }
  }    
  // distancePoit renvoie -1 dans le cas o le oint n'est pas dsignable
  public double distancePoint(double xp, double yp, boolean masquage) {
    if (!(existe) || (masquage && masque))
    return -1;
    else
    {
      VecteurDirecteur u = new VecteurDirecteur(x,y,xp,yp);
      return u.norme();
    }
  }

  public void afficheNom(Graphics2D pDC) {
    if (existe && testToile()) {
      super.afficheNom(pDC);
    }
  }

  public void positionneNom()
  {
    placeNom(x,y);
  }
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    horsEcran = !(testToile());
    positionneNom();
  }
  public void donneMotif(int mot)
  {
    motif = mot;
  }
  public void placeEn(double xn, double yn) {
    existe = true;
    x = xn;
    y = yn;
    placeNom(xn, yn);
    // Diffrence par rapport au C++ : on ne peut pas appeler positionne de CPointAncetre
    horsEcran = !(testToile());
    positionneNom();
  }

  // Java : la procdure suivante a t rajoute en java
  public void placeEn(CPointAncetre point) {
    existe = point.existe;
    x = point.x;
    y = point.y;
    placeNom(x, y);
    // Diffrence par rapport au C++ : on ne peut pas appeler positionne de CPointAncetre
    horsEcran = !(testToile());
    positionneNom();
  }

  // La procdure suivante test si lengthdplacement du point correspondant  la position
  // (xtest, ytest)  de la souris est possible et si oui renvoyant dans pointr les
  // nouvelles coordonnes et dans pointAbs.x l'abscisse pour un point li

  public boolean testDeplacement(DimensionFenetre dimfen, double xtest, double ytest,
        Point2D.Double pointr, Doub abscr)
  {
    pointr.x = xtest;
    pointr.y = ytest;
    return testFenetre(dimfen, xtest, ytest);
  }
  public boolean surDroite(CDroiteAncetre droite) {
    if (!(existe)) return false;
    VecteurDirecteur vec = new VecteurDirecteur();
    vec.x = x - droite.point_x;
    vec.y = y - droite.point_y;
    return (Kernel.colineaires(droite.vecteur,vec) && droite.appartientA(x,y));
  }
  public boolean egal(CPointAncetre pt) {
    return ((this.x == pt.x) && (this.y == pt.y));
  }

  public boolean presqueEgal(CPointAncetre A) {
   return (Kernel.zero(x - A.x) && (Kernel.zero(y - A.y)));
  }

  public final void tourne(double angle, VecteurDirecteur v)
  {
    double kc,ks;

    kc = Math.cos(angle);
    ks = Math.sin(angle);
    v.x = kc*x + ks*y;
    v.y = kc*y - ks*x;
  }
    
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est graphiquement (presque) confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne de faon  tester la validit de la construction.
   * Il s'agit simplement de comparer les positions physiques des deux points
   * pour vrifier la validit d'une construction
  **/
  public boolean coincideAvec(CElementBase p) {
    if ((p.getNature() & NatureObjet.NToutPoint) == 0) return false;
    CPointAncetre pt = (CPointAncetre) p;
    return (Kernel.zero(pt.x-x) && Kernel.zero(pt.y-y));
  }

  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    motif = inps.readByte();
  }

  /* Chargement depuis un flux java */
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    motif = inps.readByte();
  }
  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeByte(motif);
  }
  /** Fonction ajoute pour la version 2.1 en vue d'amliorer l'outil de cration d'intersection
   * Renvoie true si le point est sur la droite par dfinition
   * A redfinir pour les descendants concerns
   * */
  public boolean appartientParDefinition(CDroiteAncetre droite)
  {
    return droite.contientParDefinition(this);
  }
  /** Fonction ajoute pour la version 2.1 en vue d'amliorer l'outil de cration d'intersection
   * Renvoie true si le point est sur la droite par dfinition
   * A redfinir pour les descendants concerns
   * */
  public boolean appartientParDefinition(CCercle cercle)
  {
    return cercle.contientParDefinition(this);
  }
      
  @Override
  public String chaineDesignation() {
    return "ce commentaire";
  }
  
  /** A partir de la version 1.2.0, il est souvent ncessaire de connatre si un point est un point image par une transformation
   * Ne pouvant pas sans problme rajouter un nouveau type s'objet graphique, on cre cette fonction redfinie seulement
   * pour la classe CPointImage
   * @return
   */
  public boolean estPointImage() {
  	return false;
  }
}

