package objets;

import java.io.IOException;
import types.DimensionFenetre;
import types.NatureCalcul;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CProduit extends CCalculAncetre {
  private final static int NombreMaxiTermesProduit = 5000;
  // La liste des lments de la figure dont dpendra la somme indice
  CListeObjets listeElementsAncetres;
  
  public CVariableBornee indice;
  public CValeurDynamique valeurAMultiplier;
  // Les bornes de l'intervalle de sommation
  public CValeurAssocieeAVariable indiceMin;
  public CValeurAssocieeAVariable indiceMax;
  double valeurProduit;
  /** Creates a new instance of CSomme */
  public CProduit() {
  }
  
  // Constructeur pour clonage
  public CProduit(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    String nomCalculInit, CVariableBornee indiceInit, CValeurDynamique valeurAMultiplierInit,
    CValeurAssocieeAVariable indiceMinInit, CValeurAssocieeAVariable indiceMaxInit) {
    super(liste, impProtoInit, estElementFinalInit,  nomCalculInit);
    indice = indiceInit;
    valeurAMultiplier = valeurAMultiplierInit;
    indiceMin = indiceMinInit;
    indiceMax = indiceMaxInit;
    // Il faut crer la liste des lments dont dpend le point  tracer
    // Cette liste sera finie d'tre valide dans setClone()
    listeElementsAncetres = new CListeObjets();
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(indice);
    int ind2 = listeSource.indexOf(valeurAMultiplier);
    int ind3 = listeSource.indexOf(impProto);    
    CValeurAssocieeAVariable indiceMinClone = (CValeurAssocieeAVariable) indiceMin.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable indiceMaxClone = (CValeurAssocieeAVariable) indiceMax.getClone(listeSource, listeCible);
    CElementBase ptelb = new CProduit(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CVariableBornee)(listeCible.get(ind1, CVariableBornee.class)),
      (CValeurDynamique)listeCible.get(ind2, CValeurDynamique.class), indiceMinClone, indiceMaxClone);
    if (listeCible.getClass() != CPrototype.class) {    
	    ((CSomme)ptelb).listeElementsAncetres.setImage(listeElementsAncetres, listeProprietaire,
	      ptelb.listeProprietaire);
    }
    return ptelb;
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NProduitIndice;
  }
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || indice.dependDe(p) || valeurAMultiplier.dependDe(p)
    || indiceMin.dependDe(p) || indiceMax.dependDe(p);
  }
  
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || indice.dependDePourBoucle(p) || valeurAMultiplier.dependDePourBoucle(p)
    || indiceMin.dependDePourBoucle(p) || indiceMax.dependDePourBoucle(p);
  }
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	indiceMin.positionne(infoRandom, dimfen);
  	indiceMax.positionne(infoRandom, dimfen);
    existe = indiceMin.existe() && indiceMax.existe();
    if (!existe) return;
    // On arrondit les indices de dpart et de fin  l'entier le plus proche
    double indmin = Math.floor(indiceMin.rendValeur(infoRandom) + 0.5);
    double indmax = Math.floor(indiceMax.rendValeur(infoRandom) + 0.5);
    existe = (indmin <= indmax) && ((indmax - indmin) < NombreMaxiTermesProduit) ;
    if (!existe) return;
    double sauveIndice = indice.rendValeur(infoRandom);
    valeurProduit = 1;
    //  revoir : faut-il que la variable de boucle soit double ?
    for (double i = indmin; i <= indmax; i++)
    {
      indice.valeurActuelle = i;
      listeElementsAncetres.positionne(infoRandom, dimfen);
      existe = existe && valeurAMultiplier.existe;
      valeurProduit = valeurProduit * valeurAMultiplier.rendValeur(infoRandom);
    }
    indice.donneValeur(sauveIndice);
    listeElementsAncetres.positionne(infoRandom, dimfen);    
  }
  
  @Override
  public double rendValeur(boolean infoRandom) {
    return valeurProduit;
  }
  
  public void etablitListeElementsAncetres() {
  	listeElementsAncetres.retireTout();
    int indiceValeurASommer = listeProprietaire.indexOf(valeurAMultiplier);
    for (int i = 0; i <= indiceValeurASommer; i++) {
      CElementBase ptelb = (CElementBase)(listeProprietaire.get(i));
      if ((ptelb != indice) && (valeurAMultiplier.dependDe(ptelb)) && (ptelb.dependDe(indice)))
        listeElementsAncetres.add(ptelb);
    }  	
  }
  
  @Override
  public void metAJour() {
  	super.metAJour();
  	etablitListeElementsAncetres();
  }
  
   
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    // Il faut crer la liste des lments dont dpend le point  tracer
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la liste mre
    listeElementsAncetres = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
    int ind1 = inps.readCInt();
    indice = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    int ind2 = inps.readCInt();
    valeurAMultiplier = (CValeurDynamique) (list.get(ind2,CValeurDynamique.class));
    indiceMin = new CValeurAssocieeAVariable();
    indiceMin.chargeDepuisFlux(inps, list);
    indiceMax = new CValeurAssocieeAVariable();
    indiceMax.chargeDepuisFlux(inps, list);
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (list.getClass() != CPrototype.class) {
    	etablitListeElementsAncetres();
    }    
  }
  /* Lecture depuis dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    // Il faut crer la liste des lments dont dpend le point  tracer
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la liste mre
    listeElementsAncetres = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
    int ind1 = inps.readInt();
    indice = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    int ind2 = inps.readInt();
    valeurAMultiplier = (CValeurDynamique) (list.get(ind2,CValeurDynamique.class));
    indiceMin = new CValeurAssocieeAVariable();
    indiceMin.read(inps, list);
    indiceMax = new CValeurAssocieeAVariable();
    indiceMax.read(inps, list);
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    int indiceValeurASommer = list.indexOf(valeurAMultiplier);
    if (list.getClass() != CPrototype.class) {
      for (int i = 0; i <= indiceValeurASommer; i++) {
        CElementBase ptelb = (CElementBase)(list.get(i));
        if ((ptelb != indice) && (valeurAMultiplier.dependDe(ptelb)) && (ptelb.dependDe(indice)))
          listeElementsAncetres.add(ptelb);
      }
    }      
  }
  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(indice);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(valeurAMultiplier);
    oups.writeInt(ind2);
    indiceMin.write(oups, list);
    indiceMax.write(oups, list);
  }
}