/*
 * CSegment.java
 *
 * Created on 4 juillet 2004, 11:33
 */

/**
 *
 * @author  Yves
 * @version 
 */
package objets;

import java.awt.*;
import java.io.*;
import java.awt.geom.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

public class CSegment extends CDroite {
  public CPointAncetre point1, point2; // Les deux extrmits
  private Point2D.Double pointr = new Point2D.Double();
  /** Creates new CSegment */
  public CSegment() {
  }
  
  /* Constructeur pour les segments internes*/
  public CSegment(CListeObjets liste, CPointAncetre point1Init, CPointAncetre point2Init) {
    point1 = point1Init;
    point2 = point2Init;
    listeProprietaire = liste;
  }

  // Constructeur pour clonage
  public CSegment(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit, int styleInit,
      CPointAncetre point1Init, CPointAncetre point2Init) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
    point1 = point1Init;
    point2 = point2Init;
  }
  
  /* Fonction donnant une valeur aux pointeurs des extrmits (utilis dans l'outil
   * de cration d'image par une translation pour levecteur clignotatnt)
   */
  public void donneExtremites(CPointAncetre pt1, CPointAncetre pt2) {
    point1 = pt1;
    point2 = pt2;
  }
  
  /** Constructeur plus court pour certains objets visuels */
  public CSegment(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    CPointAncetre point1Init, CPointAncetre point2Init) {
    super(liste, impProtoInit, estElementFinalInit, Couleur.TraduitCouleur(1), false, StyleTrait.traitPointille);
    point1 = point1Init;
    point2 = point2Init;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(point1);
    int ind2 = listeSource.indexOf(point2);  
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CSegment(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, masque,style, 
      (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)));
    return ptelb;
  }

  @Override
  public String complementInfo() {
    return textes.Chaines.chinfo41;
  }
  
  // Inutile de redfinir setClone. Travail dj fait par CDroite
  
  @Override
  public String infoPourHistorique() {
    return textes.Chaines.chinfo9;
  }
  

  @Override
  public void ajouteAntecedents(CListeObjets liste) {
    liste.add(point1);
    liste.add(point2);
  }  
  
  public boolean dependDe(CElementBase p) {
	return super.dependDe(p) || point1.dependDe(p) || point2.dependDe(p);
  }
  
  public boolean dependDePourBoucle(CElementBase p) {
	return ((p ==this) || point1.dependDePourBoucle(p) || point2.dependDePourBoucle(p));
  }

  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = (point1.existe) && (point2.existe);
    if (!(existe)) return;
    point_x = point1.x;
    point_y = point1.y;
    vecteur.setVecteur(point1, point2);
    xext1 = point1.x;
    yext1 = point1.y;
    xext2 = point2.x;
    yext2 = point2.y;
  }
  
  /* fonction utilise pour les segments internes aux objets */
  public void positionne() {
    existe = (point1.existe) && (point2.existe);
    if (!(existe)) return;
    point_x = point1.x;
    point_y = point1.y;
    vecteur.setVecteur(point1, point2);
    xext1 = point1.x;
    yext1 = point1.y;
    xext2 = point2.x;
    yext2 = point2.y;    
  }
 
    // Modifi pour la version 1.9.5 car trace() ne doit plus utiliser de rfrence  des pointeurs
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    if (!vecteur.nonNul()) return;
    int styleTrait;
    if (point1.testToile() && point2.testToile())
    {
    	if (masque) styleTrait = StyleTrait.traitPointille;
    	else styleTrait = style;

      // Certains segments servent de faon interne et ne sont pas associs  une liste
      pDC.setStroke(Pinceau.CreePinceau(styleTrait));
      pDC.setColor(couleur);
      pDC.drawLine((int)Math.round(xext1),(int)Math.round(yext1),
        (int)Math.round(xext2),(int)Math.round(yext2));
    }
  }

  public boolean appartientA(double x, double y) {
    if (!existe) return false;
    else return Kernel.appartientSegment(x,y,xext1, yext1, xext2, yext2);
  } 
  
  // Rajout pour la version 2.0

  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * A t modifi par rapport  la version C++
  **/
  public boolean confonduAvec(CElementBase p) {
    CSegment pt;
    if (p.getClass() == this.getClass()) {
      pt = (CSegment) p;
      return (
        ((point1 == pt.point1) && (point2 == pt.point2))
        ||
        ((point1 == pt.point2)  && (point2 == pt.point1))
        );
    }
    else return false;
  }        
  /** distancePoint renvoie -1 dans le cas o le l'objet n'est pas dsignable 
   Modifie pour la version 1.9.5 car  cause des lieux d'objets,
   cette fonction ne doit plus utiliser de pointeurs sur point1 et point2
  **/
  public double distancePoint(double xp, double yp, boolean masquage) {
    if (!(existe) || (masquage && masque))
    return -1;
    else
    {
      VecteurDirecteur vec = new VecteurDirecteur(xext1,  yext1, xext2, yext2);
      double n = vec.norme();
      if (Kernel.zero(n)) return Kernel.distancePointPoint(xext1, yext1, xp, yp);
      else {
        Kernel.projetteOrtho(xp, yp, xext1, yext1, vec, pointr);
        if (appartientA(pointr.x,pointr.y)) {
          return Kernel.distancePointPoint(xp,yp,pointr.x, pointr.y);
        }
        else return -1;
      }      
    }
  }
  
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (point1 == ancienPoint) point1 = nouveauPoint;
    if (point2 == ancienPoint) point2 = nouveauPoint;
  }
  
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est graphiquement (presque) confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne de faon  tester la validit de la construction.
  **/
  public boolean coincideAvec(CElementBase p) {
    if (p.getNature() != NatureObjet.NSegment) return false;
    CSegment seg = (CSegment) p;
    return ((point1.coincideAvec(seg.point1) && (point2.coincideAvec(seg.point2)))
      || (point1.coincideAvec(seg.point2) && (point2.coincideAvec(seg.point1))));
  }  

  @Override
  public long getNature() {
    return NatureObjet.NSegment;
  }
  
  @Override
  public String chaineDesignation() {
    return textes.Chaines.designationSegment;
  }  

  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list)
    throws IOException {
     super.chargeDepuisFlux(inps, list);
     int ind1 = inps.readCInt();
     int ind2 = inps.readCInt();
     point1 = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
     point2 = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }
  
  /* Lecture depuis dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    point1 = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    point2 = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }
  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(point1);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(point2);
    oups.writeInt(ind2);
  }
  
  /**
   * Fonction rajoute pour la version 2.1 et servant  savoir si un point
   * est par dfinition sur une droite. Renverra true par exemple
   * si la droite est dfinie par deux points et le point est un de ces deux points
   * ou si le point est li  la droite
   * A t rajout pour amliorer l'outil d'intersection
   * A redfinir pour les descendants quand ncessaire
   * */
  public boolean contientParDefinition(CPointAncetre po)
  {
    return (point1 == po) || (point2 == po);
  }
  
  @Override
  public boolean estParalleleParDefinition(CDroiteAncetre dte){
  	return (dte == this) || (point1.appartientParDefinition(dte) && point2.appartientParDefinition(dte))
		|| dte.formentDroiteParallele(point1, point2);
  }
  
}
