/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package objets;

import types.DimensionFenetre;
import types.ErreurCalculException;
import types.NatureCalcul;

//Suite rcurrente relle de la forme u(n+1)=f[n,u(n)] o f est une fonction relle de deux variables
public class CSuiteRec2 extends CSuiteRec {

  /** Creates new CSuiteRec */
  public CSuiteRec2() {
  }
  
  /**
   * Constructeur pour cration d'objets
   * @param liste
   * @param nombreTermesInit
   * @param premierTermeInit
   */
  public CSuiteRec2(CListeObjets liste, CValeurAssocieeAVariable nombreTermesInit, CValeurAssocieeAVariable premierTermeInit) {
  	super(liste, nombreTermesInit, premierTermeInit);
  }
  
  /** Constructeur pour clonage
   * 
   * @param liste
   * @param impProtoInit
   * @param estElementFinalInit
   * @param nomCalculInit
   * @param fonctionAssocieeInit
   * @param nombreTermesInit
   * @param premierTermeInit
   */
  public CSuiteRec2(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    String nomCalculInit, CFoncNVar fonctionAssocieeInit,
    CValeurAssocieeAVariable nombreTermesInit, CValeurAssocieeAVariable premierTermeInit) {
    super(liste, impProtoInit, estElementFinalInit, nomCalculInit, fonctionAssocieeInit, nombreTermesInit, premierTermeInit);
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(fonctionAssociee);
    int ind2 = listeSource.indexOf(impProto);
    CValeurAssocieeAVariable premierTermeClone =
      (CValeurAssocieeAVariable) premierTerme.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable nombreTermesClone =
      (CValeurAssocieeAVariable) nombreTermes.getClone(listeSource, listeCible);
    CElementBase ptelb = new CSuiteRec2(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CFoncNVar)listeCible.get(ind1, CFoncNVar.class), nombreTermesClone,
      premierTermeClone);
    return ptelb;
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double u,v;
    nombreTermes.positionne(infoRandom, dimfen);
    premierTerme.positionne(infoRandom, dimfen);
    existe = fonctionAssociee.existe && premierTerme.existe() && nombreTermes.existe();
    if (!existe) return;
    double dnbt = Math.floor(nombreTermes.rendValeur(false) + 0.5);
    existe = (dnbt >= 1) && (dnbt <= NombreMaxiTermesSuiteRec);
    if (!existe) return;
    if ((int) dnbt != nbt) {
      nbt = (int) dnbt;
      metAJourNombreTermes();
    }
    
    // Modifi version 3.1.2 pour autoriser la remise  jour des valeurs renvoyes par rand
    // u = premierTerme.rendValeur(infoRandom);
    u = premierTerme.rendValeur(true);
    valeurs[0] = u;
    indiceDernierTermeExistant = 0;
    int i = 1;
    while (i < nbt) {
      try {
        listeProprietaire.initialiseNombreIterations();
        // v = fonctionAssociee.calcul.resultatFonction(infoRandom, u);
        double [] arg = new double[2];
        arg[0] = i - 1;
        arg[1] = u;
        v = fonctionAssociee.calcul.resultatFonction(true, arg);
        if (Double.isNaN(v) || Double.isInfinite(v)) {
          break;
        }
        u = v;
        valeurs[i] = u;
        i++;
      } catch (ErreurCalculException e) {
        break;
      }
    }
    i--;
    indiceDernierTermeExistant = i;
  }

  @Override
  public String infoPourHistorique() {
  	String ch = getString("chinfo185") + "\n" + getString("chinfo84") + " " + fonctionAssociee.nom() + "\n" +
  	getString("chinfo85") + " " + premierTerme.chaineInfo() + "\n" +
  	getString("chinfo86") + " " + nombreTermes.chaineInfo();
    if (!existe) ch = ch + "\n" + getString("ch18");
    return ch;
  }

 @Override
 public long getNatureCalcul() {
 	return NatureCalcul.NSuiteRecurrenteReelle2;
 }

}
