/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Cursor;
import java.awt.Point;
import Applet.AppletMathGraph32;
import objets.CBarycentre;
import objets.CElementGraphique;
import objets.CTransformation;
import types.NatureObjet;
import dialogs.MtgDlg;

public class OutilModifierObjetGraphique extends Outil {
	public CElementGraphique objetDesigne; // public car accessible par le dialogue d'historique

	public OutilModifierObjetGraphique(AppletMathGraph32 appletInit) {
		super(appletInit, true, AppletMathGraph32.iconModifierObjetGraphique);
	}
	
  public String toolTip() {
    return getString("toolTip36");
  }	

  @Override
	public void select() {
		super.select();
		appletMere.outilPointageActif = appletMere.outilPointageModification;
		appletMere.outilActif.designationDe(NatureObjet.NToutObjet);
		appletMere.indication(getString("modificationObjetGraphique1"));
	}
	
	@Override
  public Cursor curseurUtilise() {
		return AppletMathGraph32.modifCursor;
	}
	

	@Override
	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
		if (elg.getNature() == NatureObjet.NBarycentre) {
			annuleClignotement();
			deselect();
			appletMere.outilActif = appletMere.outilModifierBarycentre;
			appletMere.outilModifierBarycentre.select((CBarycentre) elg);
		}
		else {
			objetDesigne = elg;
	    ajouteClignotementDe(elg);
			elg.ajouteAntecedents(appletMere.listeClignotante);
	    appletMere.videDesignation();
	    resetClignotement();
			creeObjet();
		}
	}

	@Override
	public void creeObjet() {
		if (objetDesigne.antecedentDirect().getNature() == NatureObjet.NTransformation) {
			// Cas o l'objet cliqu est l'image d'une autre objet par une transformation
			CTransformation trans = (CTransformation)objetDesigne.antecedentDirect();
			trans.modificationImage(appletMere, objetDesigne);
		}
		else {
			MtgDlg dlg = objetDesigne.dialogueModification(appletMere);
			dlg.setLocationRelativeTo(appletMere);
			dlg.setVisible(true);
			if (dlg.retourOK()) {
			  appletMere.gestionnaire.enregistreFigureEnCours(objetDesigne.informationModification());
			  appletMere.listePr.metAJourObjetsDependantDe(objetDesigne);
			  appletMere.repaintFigureAvecCalcul();
			}
		}
		annuleClignotement();
		select();
	}
	/**
	 * Fonction appele par le dialogue d'historique
	 */
	public void creeObjetPourHistorique() {
		if (objetDesigne.antecedentDirect().getNature() == NatureObjet.NTransformation) {
			// Cas o l'objet cliqu est l'image d'une autre objet par une transformation
			CTransformation trans = (CTransformation)objetDesigne.antecedentDirect();
			trans.modificationImage(appletMere, objetDesigne);
		}
		else {
			MtgDlg dlg = objetDesigne.dialogueModification(appletMere);
			dlg.setLocationRelativeTo(appletMere);
			dlg.setVisible(true);
			if (dlg.retourOK()) {
			  appletMere.gestionnaire.enregistreFigureEnCours(objetDesigne.informationModification());
			  appletMere.listePr.metAJourObjetsDependantDe(objetDesigne);
			  appletMere.repaintFigureAvecCalcul();
			}
		}
	}
	
}
