/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 *
 * @author Yves
 * Outil de pointage utilis pour la capture de points mobiles
 */
package outils;

import java.awt.*;
import java.awt.geom.*;
import dialogs.PremierDernierDlg;
import objets.*;
import types.*;
import Applet.*;

public class OutilPointageCapt extends OutilPointage {
  /* pointPredesigne contient un point mobile qui a t choisi lors d'un dialogue de choix
   * de type ou du type premier-dernier
  */
  private CPointAncetrePointsMobiles pointPredesigne;
  public boolean deplacementPointEnCours;
  public CPointAncetrePointsMobiles elementCapture;
  // Le champ x de pointabs contient la nouvelle abscisse d'un point
  // li qui doit tre dplac
  public  Doub pointabs = new Doub();
  // pointres contient les nouvelles coordonnes d'un point libre qui doit tre dplac
  public  Point2D.Double pointres = new Point2D.Double();
  
  /** Creates a new instance of OutilPointageCapt */
  public OutilPointageCapt(AppletMathGraph32 appletMere) {
    super(appletMere);
    // reset();
  }
  
  public void reset() {
    super.reset();
    deplacementPointEnCours = false;
    pointPredesigne = null;
    elementCapture = null;
  }
  
  // Ajout pout version 2.0
  @Override
  public void actionDansPaint(AppletMathGraph32 applet) {
  	if (deplacementPointEnCours) {
			applet.paneFigure.effacegBuffer();
			applet.paneFigure.afficheToutDansgbuffer();
			aRedessinerDansPaint = false;
  	}
  }
    
  public void mouseClicked(java.awt.event.MouseEvent mouseEvent) {
  }
  
  public void mouseMoved(java.awt.event.MouseEvent mouseEvent) {
    MtgPaneFigure paneFigure = appletMere.paneFigure;
    CListeObjets liste = appletMere.listePr;
    InfoProx info = appletMere.infoProx;
    int nbObjetsProches = liste.procheDe
      (NatureObjet.NPointMobile | NatureObjet.NMacro, mouseEvent.getPoint(), info,
      appletMere.listeExclusionDesignation);
    boolean designationDejaAffichee = (appletMere.commentaireDesignation.chaineCommentaire.length() != 0);
      // true si une dsignation est dj affiche prs du pointeur
    if (nbObjetsProches > 0) {
      String ch = new String ("");
      if (info.premierPointVoisin != null) {
        // ch = getString("capturer"); // Supprim version 4.7.4
        setActiveCursor(AppletMathGraph32.handCursor);
      }
      else {
        ch = getString("executer");
        setActiveCursor(AppletMathGraph32.defaultCursor);
      }
      appletMere.commentaireDesignation.donneChaine(ch);
      appletMere.commentaireDesignation.placeEn(0, 0);
      if (appletMere.macroEnCours == null) {
        aRedessinerDansPaint = true;
        paneFigure.repaint();
      }
    }
    else {
      setActiveCursor(AppletMathGraph32.crossHairCursor);
      appletMere.videDesignation();
      if ((appletMere.macroEnCours == null) && designationDejaAffichee) {
        aRedessinerDansPaint = true;
      	paneFigure.repaint();
      }
    }
  }
  public void mousePressed(java.awt.event.MouseEvent mouseEvent) {
    if (elementCapture != null) return;
    MtgPaneFigure paneFigure = appletMere.paneFigure;
    CListeObjets liste = appletMere.listePr;
    InfoProx info = appletMere.infoProx;
    int nbObjetsProches = info.nombreObjetsProches;
    if (nbObjetsProches > 0) {
      if (info.premierPointVoisin != null)
      { // A revoir si une macro est en cours d'excution
        // Priorit aux points mobiles
        if (info.nombrePointsVoisins == 1) {
          elementCapture = (CPointAncetrePointsMobiles) info.premierPointVoisin;
        }
        else {
          if (pointPredesigne != null) {
            if (pointPredesigne.distancePoint(mouseEvent.getX(), mouseEvent.getY(), true) <= 4) {
              elementCapture = pointPredesigne;
              pointPredesigne = null;
            }
          }
          else {
            PremierDernierDlg dlg = new PremierDernierDlg(appletMere);
            dlg.setLocationRelativeTo(appletMere);
            dlg.setVisible(true);
            if (!dlg.retourOK()) return;
            // On regarde quelle a t la rponse de l'utilisateur
            if (dlg.getPremierChoisi()) pointPredesigne = (CPointAncetrePointsMobiles) info.premierPointVoisin;
            else pointPredesigne = (CPointAncetrePointsMobiles) info.dernierPointVoisin;
            return;
          }
        }
        naturePointCapture = elementCapture.getNature();
        appletMere.listeDependantsDansPrincipale.retireTout();
        appletMere.listeDependantsDansPrincipale.ajouteObjetsDependants
          (elementCapture, liste);
        if ((appletMere.macroParDefaut != null) && (paneFigure.gbuffer != null)) 
          paneFigure.effacegBuffer();
      }
      else { // Cas d'une macro clique
        // L'objet macro est le NatureObjet.indiceMacro ime objet graphique
        CMacro macroCliquee = (CMacro) info.infoParType[NatureObjet.indiceMacro].premierVoisin;
        if (appletMere.macroEnCours == null) { // Si aucune macro n'est dj en cours d'excution
                                    // on lance l'excution de la macro clique si possible
          // Si une macro d'animation est en cours on n'autorise que l'excution des macros
          // sans animation
          if (appletMere.macroAnimation != null) {
            if (macroCliquee.typeAnimation() != CMacro.sansAnimation) return;
          }
          if (appletMere.macroAnimation == macroCliquee) return;
          if (macroCliquee.executionPossible()) {
            appletMere.macroEnCours = macroCliquee;
            appletMere.macroEnCours.initialise(appletMere);
            appletMere.macroEnCours.macroEnCours().actionAvantExecution(appletMere);
            // Si la macro utilise un thread, il faut le crer.
            //  revoir
            appletMere.macroEnCours.macroEnCours().initialiseInterruptions(appletMere);

              //macroEnCours.execute(this, infRand, dimf);
            if (appletMere.macroEnCours.macroEnCours().typeAnimation() == CMacro.sansAnimation) {
              //appletMere.macroEnCours.macroEnCours().actionDansPaint(appletMere);
              paneFigure.repaint();
              // Ligne suivante rajoute pour la version 1.9.5
              // et retire pour la version 2.5.6 la ligne correspondante a t rajoute
              // dans paint de MtgPaneFigure (correction d'un bub)
              /*
              if (appletMere.macroEnCours.macroEnCours() == appletMere.macroEnCours) {
                if (!appletMere.macroEnCours.executionEnCours) appletMere.macroEnCours = null;
              }
               * */
            }
          }
          else Toolkit.getDefaultToolkit().beep();
        }
        else {
          CMacro mac = appletMere.macroEnCours.macroEnCours();
          // Si la macro en cours d'excution s'excute par un thread et si elle n'a pas fini de s'excuter
          // on n'active pas une autre macro
          if ((mac.typeAnimation() == CMacro.animationParThread) && mac.executionEnCours) return;
          // Si la macro en cours est un CMacroSuiteMacros on la fait passer  la macro suivante            
          if (appletMere.macroEnCours instanceof CMacroSuiteMacros) {
            if (mac.arretParClic()) {
              mac.termineAction(appletMere);
              ((CMacroSuiteMacros) appletMere.macroEnCours).passageMacroSuiv(appletMere);
            }
          }
          else {
            if ((mac instanceof CMacroApparition) && mac.executionEnCours) {
              // mac.actionDansPaint(cadre);
              paneFigure.repaint();
              return;
            }
            if (macroCliquee == appletMere.macroEnCours) {
              mac.termineAction(appletMere);
              if (mac.typeAnimation() == CMacro.sansAnimation) {
                mac.actionAvantExecution(appletMere);
                paneFigure.repaint();
              }
              else {
                mac.termineAction(appletMere);
                appletMere.macroEnCours = null;
                System.gc();
              }           
            }
            else {
              appletMere.macroEnCours.termineAction(appletMere);
              if (macroCliquee.executionPossible()) {
                appletMere.macroEnCours = macroCliquee;
                appletMere.macroEnCours.initialise(appletMere);
                appletMere.macroEnCours.actionAvantExecution(appletMere);
                // Si la macro utilise un thread, il faut le crer.
                appletMere.macroEnCours.initialiseInterruptions(appletMere);

                  //macroEnCours.execute(this, infRand, dimf);
                if (mac.typeAnimation() == CMacro.sansAnimation) paneFigure.repaint();
              }
              else Toolkit.getDefaultToolkit().beep();
            }
          }
        }
      }
    }
    else {  // Si aucun objet n'est proche du pointeur souris, et si une macro est en cours
            // d'excution, on annule son excution
      if (appletMere.macroEnCours != null) {
        if (appletMere.macroEnCours instanceof CMacroSuiteMacros) {
          if (appletMere.macroEnCours.macroEnCours().arretParClic()) {
            appletMere.macroEnCours.macroEnCours().termineAction(appletMere);
            ((CMacroSuiteMacros) appletMere.macroEnCours).passageMacroSuiv(appletMere);
          }
          // passageMacroSuiv redonne ventuellement un ordre paint() suivant la nature de la macro suivante
        }
        else {
          // Si la macro en cours est une macro d'apparition avec clic pour objet suivant
          // on ne la dsactive que si on est au dernier objet
          CMacro mac = appletMere.macroEnCours.macroEnCours();
          if ((mac instanceof CMacroApparition) && mac.executionEnCours) {
            mac.actionDansPaint(appletMere);
            paneFigure.repaint();
          }
          else {
            // modifi le 21/5/09
            // CMacro macEnCours = appletMere.macroEnCours;
            if (mac.arretParClic()) {
              mac.termineAction(appletMere);
              appletMere.macroEnCours = null;
            }
          }
          System.gc();
        }
      }
    }
  }
  public void mouseDragged(java.awt.event.MouseEvent mouseEvent) {
  	appletMere.videDesignation();
    MtgPaneFigure paneFigure = appletMere.paneFigure;
    int x = mouseEvent.getX();
    int y = mouseEvent.getY();
    //  revoir
    if ((x > appletMere.dimf.longueurAbscisses) || (y > appletMere.dimf.longueurOrdonnees)) {
      return;
    }
    
    if ((elementCapture != null) && (appletMere.macroEnCours == null)) {
      if (naturePointCapture == NatureObjet.NPointLie) {

        CPointLie ptPointLie = (CPointLie)(elementCapture);
        deplacementPointEnCours = ptPointLie.testDeplacement(appletMere.dimf,x,y,
          pointres,pointabs);
        if (deplacementPointEnCours) {
	        ptPointLie.donneAbscisse(pointabs.getValue());
	        appletMere.listeDependantsDansPrincipale.positionne(appletMere.infRand, appletMere.dimf);
        	aRedessinerDansPaint = true;
        	paneFigure.repaint();
        }
      }
      else {
        if (naturePointCapture == NatureObjet.NPointBase) {
          CPointBase ptbase = (CPointBase) elementCapture;
          deplacementPointEnCours = ptbase.testDeplacement(appletMere.dimf,x,y,pointres,pointabs);
          if (deplacementPointEnCours) {
  	        ptbase.placeEn(pointres.x, pointres.y);
  	        appletMere.listeDependantsDansPrincipale.positionne(appletMere.infRand, appletMere.dimf);
  	        aRedessinerDansPaint = true;
          	paneFigure.repaint();
          }
        }
        else {
        	if (naturePointCapture == NatureObjet.NPointBaseEnt) {
	          CPointBaseEnt ptbaseEnt = (CPointBaseEnt) elementCapture;
	          deplacementPointEnCours = ptbaseEnt.testDeplacement(appletMere.dimf,x,y,pointres,pointabs);
	          if (deplacementPointEnCours){
	            ptbaseEnt.placeEn(pointres.x, pointres.y);
	  	        ptbaseEnt.placeEn(pointres.x, pointres.y);
	  	        appletMere.listeDependantsDansPrincipale.positionne(appletMere.infRand, appletMere.dimf);
	  	        aRedessinerDansPaint = true;
	  	        paneFigure.repaint();
	          }
        	}
    			else {
    				if (naturePointCapture == NatureObjet.NPointInterieurPolygone) {
      				CPointInterieurPolygone pointint = (CPointInterieurPolygone) elementCapture;
  	          deplacementPointEnCours = pointint.testDeplacement(appletMere.dimf,x,y,pointres,pointabs);
  	          if (deplacementPointEnCours){
      					pointint.placeEn(pointres.x, pointres.y);
      					pointint.positionne(appletMere.infRand, appletMere.dimf);
  	  	        appletMere.listeDependantsDansPrincipale.positionne(appletMere.infRand, appletMere.dimf);
         				aRedessinerDansPaint = true;
         				paneFigure.repaint();
  	          }
	  	      }
	  				else {
	    				if (naturePointCapture == NatureObjet.NPointInterieurCercle) {
	      				CPointInterieurCercle pointint = (CPointInterieurCercle) elementCapture;
	  	          deplacementPointEnCours = pointint.testDeplacement(appletMere.dimf,x,y,pointres,pointabs);
	  	          if (deplacementPointEnCours){
	      					pointint.placeEn(pointres.x, pointres.y);
	      					pointint.positionne(appletMere.infRand, appletMere.dimf);
	  	  	        appletMere.listeDependantsDansPrincipale.positionne(appletMere.infRand, appletMere.dimf);
	         				aRedessinerDansPaint = true;
	         				paneFigure.repaint();
	  	          }
		  	      }
	  				}
  				}
        	
        }
      }
    }
  }
  
  public void mouseReleased(java.awt.event.MouseEvent mouseEvent) {
    // On sauvegarde l'tat de la figure avant capture
    if (elementCapture != null) {
    	elementCapture = null; // Version 4.2.3 : plac avant enregistreFigureEnCours qui prend du temps
    	deplacementPointEnCours = false; // et aprs un certain nombre d'appels il semble que la 
    	appletMere.listeDependantsDansPrincipale.retireTout(); // de la fonction ne soit plus excute ....
    	appletMere.gestionnaire.enregistreFigureEnCours(getString("chaineOutil93"));
    }
    CListeObjets liste = appletMere.listePr;
    InfoProx info = appletMere.infoProx;
    int nbObjetsProches = liste.procheDe
      (NatureObjet.NPointMobile | NatureObjet.NMacro, mouseEvent.getPoint(), info,
      appletMere.listeExclusionDesignation);
    if (nbObjetsProches > 0) {
      if (info.premierPointVoisin != null) setActiveCursor(AppletMathGraph32.handCursor);
      else setActiveCursor(AppletMathGraph32.defaultCursor);
    }
    else setActiveCursor(AppletMathGraph32.crossHairCursor);
    // elementCapture = null;
    // deplacementPointEnCours = false;
    // appletMere.listeDependantsDansPrincipale.retireTout();
    // S'il y a une macro par dfaut il fuat rafficher la figure pour l'excuter
    if (appletMere.macroParDefaut != null) appletMere.paneFigure.repaint();
    // On dcharge le garbage Collector
    System.gc();
  }
  
}
