package types;
/*
 * StyleRemplissage.java
 *
 * Created on 17 juillet 2004, 21:28
 */

/**
 *
 * @author  Yves
 * @version 
 */
// Classe dfinissant les diffrents types de remplissage possibles pour les surfaces
// et fournissant des objets TexturePaint servant aux remplissages
import java.awt.image.*;
import java.awt.geom.*;
import java.awt.*;
public abstract class StyleRemplissage {

  
  public static final int QuadrillageVertical = 0;
  public static final int QuadrillageHorizontal = 1;
  public static final int Quadrillage45Haut = 2;
  public static final int Quadrillage45Bas = 3;
  public static final int QuadrillagePlein = 4;
  public static final int QuadrillagePleinTransp = 5;
  public static final int QuadrillageCroise = 6;
  
  public static final TexturePaint getTexturePaint(Color couleur, int styleRemplissage, boolean basseDefinition,
  		Color couleurFond) {
    BufferedImage bi = new BufferedImage(4, 4, BufferedImage.TYPE_4BYTE_ABGR);
    WritableRaster r = bi.getRaster();
    // On spcifie les 3 composantes pour le trait et le coefficient de transparence en dernier
    int[] couleurTrait = new int[4];
    couleurTrait[0] = couleur.getRed();
    couleurTrait[1] = couleur.getGreen();
    couleurTrait[2] = couleur.getBlue();
    couleurTrait[3] = 255; // Le trait remplace ce qu'il recouvre
    int[] coulFond = new int[4];
    coulFond[0] = couleurFond.getRed();
    coulFond[1] = couleurFond.getGreen();
    coulFond[2] = couleurFond.getBlue();
    Rectangle2D.Double rect = new Rectangle2D.Double(0,0,4,4);
    
    switch (styleRemplissage) {
      case QuadrillageVertical :

        coulFond[3] = 0; // La couleur de fond est transparente pour les hachures     
        for (int i = 0; i < 4; i++) {
          for (int j = 0; j < 4; j++) {
            r.setPixel(i, j, coulFond);
          }
        }
        r.setPixel(0,0,couleurTrait);
        r.setPixel(0,1,couleurTrait);
        r.setPixel(0,2,couleurTrait);
        r.setPixel(0,3,couleurTrait);
        return new TexturePaint(bi, rect);
      case QuadrillageHorizontal :

        coulFond[3] = 0; // La couleur de fond est transparente pour les hachures
        for (int i = 0; i < 4; i++) {
          for (int j = 0; j < 4; j++) {
            r.setPixel(i, j, coulFond);
          }
        }
        r.setPixel(0,0,couleurTrait);
        r.setPixel(1,0,couleurTrait);
        r.setPixel(2,0,couleurTrait);
        r.setPixel(3,0,couleurTrait);
        return new TexturePaint(bi, rect);
      case Quadrillage45Bas :

        coulFond[3] = 0; // La couleur de fond est transparente pour les hachures
        for (int i = 0; i < 4; i++) {
          for (int j = 0; j < 4; j++) {
            r.setPixel(i, j, coulFond);
          }
        }
        r.setPixel(0,0,couleurTrait);
        r.setPixel(1,1,couleurTrait);
        r.setPixel(2,2,couleurTrait);
        r.setPixel(3,3,couleurTrait);
        return new TexturePaint(bi, rect);
      case Quadrillage45Haut :

        coulFond[3] = 0; // La couleur de fond est transparente pour les hachures
        for (int i = 0; i < 4; i++) {
          for (int j = 0; j < 4; j++) {
            r.setPixel(i, j, coulFond);
          }
        }
        r.setPixel(3,0,couleurTrait);
        r.setPixel(2,1,couleurTrait);
        r.setPixel(1,2,couleurTrait);
        r.setPixel(0,3,couleurTrait);
        return new TexturePaint(bi, rect);
      case QuadrillageCroise :

        coulFond[3] = 0; // La couleur de fond est transparente pour les hachures
        for (int i = 0; i < 4; i++) {
          for (int j = 0; j < 4; j++) {
            r.setPixel(i, j, coulFond);
          }
        }
        r.setPixel(0,0,couleurTrait);
        r.setPixel(1,1,couleurTrait);
        r.setPixel(2,2,couleurTrait);
        r.setPixel(3,3,couleurTrait);
        r.setPixel(3,0,couleurTrait);
        r.setPixel(2,1,couleurTrait);
        r.setPixel(1,2,couleurTrait);
        r.setPixel(0,3,couleurTrait);
        return new TexturePaint(bi, rect);        
      case QuadrillagePlein :

        coulFond[3] = 0; // La couleur de fond est transparente pour les pointills (version 1.6.2)
        for (int i = 0; i < 4; i++) {
          for (int j = 0; j < 4; j++) {
          	if ((i+j)%2 == 0) r.setPixel(i, j, couleurTrait);
            else r.setPixel(i, j, coulFond);
          }
        }
        return new TexturePaint(bi, rect);        

      case QuadrillagePleinTransp :

        for (int i = 0; i < 4; i++) {
          for (int j = 0; j < 4; j++) {
            r.setPixel(i, j, couleurTrait);
          }
        }
        return new TexturePaint(bi, rect);
        
      default : return null; // Ne doit pas arriver
    }
  }
}